/* global webAppCore */
/* global ManagementGame */
(function() {
    // event listeners
    window.addEventListener("load",startApp,false);

    // global hooks
    window.wacMenuOptionSelected = onMenuOptionSelected;
    
    let curTime;
    let gameLoop;
    let currentGame;
    let tmpSellResVal;
    let singleCardHolder;
    let allCardsContainer;

    let playingCatchup = false;
    let financialReportOpen = false;
    let gamePaused = false;
    let listAll = false;
    let allListed = false;
    let allowAudio = true;

    let tmpNewStorageCap = 0;
    let pendingMaxLoanAmount = 0;
    let saveCounter = 0;

    let currentSong = null;
    let lastWaitingOnContent = "";
    
    function startApp() {
        webAppCore.startApp(initApp);
    }

    function initApp() {

        if(ManagementGame.meta.customSidebarCategory) {
            if(ManagementGame.meta.customSidebarCategoryIcon) {
                webAppCore.setSidebarCategory(ManagementGame.meta.customSidebarCategory,ManagementGame.meta.customSidebarCategoryIcon);
            } else {
                webAppCore.setSidebarCategory(ManagementGame.meta.customSidebarCategory);
            }
        } else {
            webAppCore.setSidebarCategory("Idle Games","idle.png");
        }

        let audioSetting = webAppCore.getVar("mbe-allow-audio","1");

        if(audioSetting == "1") {
            allowAudio = true;
        } else {
            allowAudio = false;
        }

        if(webAppCore.isTV()) {
            document.getElementById("wrapper").className = "tvwrapper";
        }

        console.log("companies: " + ManagementGame.companies.length);
        console.log("techs: " + ManagementGame.technologies.length);

        initTheme();
        
        
        const d = new Date();
        curTime = d.getTime();
        
        gooseUpMenu();

        setupHandlers();
        
        webAppCore.handlePauseFunction(onPause);
        webAppCore.handleResumeFunction(onResume);
        webAppCore.handleBackFunction(null);
        webAppCore.setOnImageImportListener(onImage);
        webAppCore.setApeAppsStatusListener(onApeAppsStatusChange);
        webAppCore.setGamepadFunctions(onPadDown,onPadUp);
        webAppCore.setOnPremiumListener(populateBusinessListing);

        populateBusinessListing();
        setupResourceDisplays();
        loadGame();

    }

    function initTheme() {

        webAppCore.setTheme(ManagementGame.meta.backgroundColor);

        document.body.style.backgroundColor = ManagementGame.meta.backgroundColor;
        document.body.style.color = ManagementGame.meta.foregroundColor;

        document.getElementById("wrapper").style.backgroundImage = "url('" + ManagementGame.meta.backgroundImage + "')";

        if(!ManagementGame.meta.backgroundTiled) {
            document.getElementById("wrapper").style.backgroundSize = "cover";
            document.getElementById("wrapper").style.backgroundPosition = "center";
            document.getElementById("wrapper").style.backgroundRepeat = "no-repeat";
        }

        document.getElementById("revenueReport").style.backgroundColor = ManagementGame.meta.reportBackgroundColor;
        document.getElementById("revenueReport").style.color = ManagementGame.meta.reportForegroundColor;

        if(ManagementGame.meta.reportBackgroundImage) {
            document.getElementById("revenueReport").style.backgroundImage = "url('" + ManagementGame.meta.reportBackgroundImage + "')";

            if(!ManagementGame.meta.reportBackgroundTiled) {
                document.getElementById("revenueReport").style.backgroundSize = "cover";
                document.getElementById("revenueReport").style.backgroundPosition = "center";
                document.getElementById("revenueReport").style.backgroundRepeat = "no-repeat";
            }
        }

        let newStyle = document.createElement("style");
        newStyle.appendChild(document.createTextNode("@font-face { font-family: '" + ManagementGame.gameFont.family + "'; src: url('" + ManagementGame.gameFont.url + "') format('" + ManagementGame.gameFont.format + "'); } #timeReadout,#population,.resourceDisplayItemSpan,#businessCard,#noBusinessWarning,#businessListing,#revenueReport,#storageMeterLabel { font-family: '" + ManagementGame.gameFont.family + "'; }"));
        document.head.appendChild(newStyle);
    }

    function gooseUpMenu() {

        webAppCore.clearMenu();

        //Set up the menu
        webAppCore.addMenuOption("New Game","wacMenuOptionSelected","page","#F44336","new");

        webAppCore.addMenuOption("Accounting","wacMenuOptionSelected","calculator",null,"payroll");
        
        if(ManagementGame.crimes.length > 0) {
            webAppCore.addMenuOption("Legal","wacMenuOptionSelected","gavel",null,"legal");
        }
        
        webAppCore.addMenuOption("Human Resources","wacMenuOptionSelected","people",null,"employees");

        if(ManagementGame.technologies && ManagementGame.technologies.length > 0) {
            webAppCore.addMenuOption("Technology","wacMenuOptionSelected","library",null,"tech");
        }

        if(ManagementGame.resources && ManagementGame.resources.length > 0) {
            webAppCore.addMenuOption("Resources","wacMenuOptionSelected","fourbars",null,"resources");
        }

        webAppCore.addMenuOption("Rename Business","wacMenuOptionSelected","rename",null,"rename");
        webAppCore.addMenuOption("Sell Business","wacMenuOptionSelected","clear","#F44336","sell");
        
        if(ManagementGame.meta.communityUrl != null) {
            if(!webAppCore.isTV()) {
                webAppCore.addMenuOption("Community","wacMenuOptionSelected","people",null,"community");
            }
        }

        if(ManagementGame.externalLink && ManagementGame.externalLink.name) {
            if(!webAppCore.isTV()) {
                webAppCore.addMenuOption(ManagementGame.externalLink.name,"wacMenuOptionSelected",ManagementGame.externalLink.icon,null,"externallink",null,true);
            }
        }

        if(allowAudio) {
            webAppCore.addMenuOption("Turn Sound Off","wacMenuOptionSelected","volume","#F44336","soundoff");
        } else {
            webAppCore.addMenuOption("Turn Sound On","wacMenuOptionSelected","volume","#4CAF50","soundon");
        }
    }
    
    function onImage() {
        // who cares
    }
    
    function onBack() {
        webAppCore.handleBackFunction(null);
        
        document.getElementById("revenueReport").style.display = "none";

    }
    
    function updateListAllDisplay() {
        if(!listAll) {
            allCardsContainer.innerHTML = "";
            allCardsContainer.style.display = "none";
            singleCardHolder.style.display = "block";
            return;
        }

        singleCardHolder.style.display = "none";
        allCardsContainer.style.display = "block";
        
        if(allListed) {
            return;
        }
        
        for(let i = 0; i < currentGame.businesses.length; i++) {
            let newHolder = document.createElement("div");
            newHolder.className = "cardListingHolder";
            allCardsContainer.appendChild(newHolder);
        }
        
        allListed = true;
        
    }

    let businessListItems;

    function showTechWindow() {
        if(!currentGame) {
            webAppCore.showToast("Please build your first business to see available tech!");
            return;
        }

        let isPremium = webAppCore.getIsPremium();

        let menuOptions = [];

        for(let i = 0; i < ManagementGame.technologies.length; i++) {

            if(currentGame.unlockedTechs.indexOf(ManagementGame.technologies[i].name) > -1) {
                continue;
            }

            if(ManagementGame.technologies[i].requiresTech && ManagementGame.technologies[i].requiresTech.length > 0) {

                let canBuild = true;

                for(let j = 0; j < ManagementGame.technologies[i].requiresTech.length; j++) {
                    let techName = ManagementGame.technologies[i].requiresTech[j];

                    if(currentGame.unlockedTechs.indexOf(techName) < 0) {
                        canBuild = false;
                    }
                }

                if(!canBuild) {
                    continue;
                }
            }

            let preColor = null;
            let preText = "Cost: " + numberToCash(ManagementGame.technologies[i].cost);
            let useTag = ManagementGame.technologies[i];

            if(ManagementGame.technologies[i].premium && !isPremium) {
                preColor = "#FFC107";
                preText = "Requires Premium";
                useTag = "reqpremium";
            } else {
                if(ManagementGame.technologies[i].cost > currentGame.cash) {
                    preColor = "#F44336";
                }
            }

            let usedesc = null;

            if(ManagementGame.technologies[i].description) {
                usedesc = ManagementGame.technologies[i].description;
            }

            menuOptions.push({
                "title":        ManagementGame.technologies[i].name,
                "title-color":  null,    
                "description":  usedesc,
                "status":       preText,
                "status-color": preColor,
                "icon":         "images/tech/" + ManagementGame.technologies[i].icon,
                "tag":          useTag
            });
        }

        if(menuOptions.length == 0) {
            webAppCore.showToast("Nothing to unlock right now!");
            return;
        }

        webAppCore.presentNativeList("Technology",menuOptions,onTechSelected,"Dismiss");
    }

    function onTechSelected(tag) {
        if(!tag) {
            return;
        }

        if(tag == "reqpremium") {
            showPremiumDialog();
            return;
        }

        if(parseInt(tag.cost) > currentGame.cash) {
            webAppCore.showToast("You can't afford this yet!");
            return;
        }

        if(currentGame.jailDaysRemaining > 0) {
            webAppCore.showAlertWithTitle("You cannot conduct research while you are in jail!  You still have " + currentGame.jailDaysRemaining + " days remaining.","In Jail");
            return;
        }

        currentGame.cash -= parseInt(tag.cost);

        currentGame.unlockedTechs.push(tag.name);
        populateBusinessListing();

        webAppCore.showToast("Unlocked " + tag.name + "!");
    }

    function populateBusinessListing() {

        if(!currentGame) {
            return;
        }

        document.getElementById("businessCardMeterReadoutResources").innerHTML = "";

        let i;

        if(ManagementGame.resources) {
            for(i = 0; i < ManagementGame.resources.length; i++) {
                let resItem = document.createElement("img");
                resItem.draggable = false;
                resItem.title = ManagementGame.resources[i].name;
                resItem.src = "images/resources/" + ManagementGame.resources[i].icon;
                resItem.id = "meter-readout-resource-" + ManagementGame.resources[i].name;
                document.getElementById("businessCardMeterReadoutResources").appendChild(resItem);
            }
        }

        businessListItems = [];

        let isPremium = webAppCore.getIsPremium();

        let useDesc,j,company;

        for(i = 0; i < ManagementGame.companies.length; i++) {

            company = ManagementGame.companies[i];

            useDesc = null;

            let canBuild = true;

            if(company.requiresTech && company.requiresTech.length > 0) {

                

                for(j = 0; j < company.requiresTech.length; j++) {
                    let techName = company.requiresTech[j];

                    if(currentGame.unlockedTechs.indexOf(techName) == -1) {
                        canBuild = false;
                    }
                }

                
            }

            if(!canBuild) {
                continue;
            }

            let preColor = null;
            let preText = "Cost: " + numberToCash(company.cost);
            let useTag = company;

            if(company.premium && !isPremium) {
                preColor = "#FFC107";
                preText = "Requires Premium";
                useTag = "reqpremium";
            }

            if(company.description) {
                useDesc = company.description;
            }

            let overflowItems = [];

            if(company.oneOffJob) {
                overflowItems.push({
                    pre:    "One Time Job",
                    color:  "#00ccff"
                });
            }

            if(company.workers > 0) {
                overflowItems.push({
                    pre:    "Workers",
                    icon:   "images/ui/worker.png",
                    post: numberWithCommas(company.workers)
                });
            }

            if(company.storageMod && company.storageMod > 0) {
                overflowItems.push({
                    pre:    "Storage",
                    icon:   "images/ui/storage.png",
                    post: numberWithCommas(company.storageMod)
                });
            }

            if(company.resourceGen && company.resourceGen.input.length > 0) {
                for(j = 0; j < company.resourceGen.input.length; j++) {

                    

                    let rdef = getResourceDefByName(company.resourceGen.input[j].resource);

                    if(!currentGame.resources[rdef.name] || !currentGame.resources[rdef.name].isUnlocked) {
                        canBuild = false;
                    }

                    if(rdef) {
                        overflowItems.push({
                            pre:    "Requires",
                            icon:   "images/resources/" + rdef.icon,
                            post: rdef.name + " x" + numberWithCommas(company.resourceGen.input[j].amount)
                        });
                    }
                }
            }

            if(company.resourceGen && company.resourceGen.output.length > 0) {
                for(j = 0; j < company.resourceGen.output.length; j++) {
                    let rdef2 = getResourceDefByName(company.resourceGen.output[j].resource);

                    if(rdef2) {
                        overflowItems.push({
                            pre:    "Produces",
                            icon:   "images/resources/" + rdef2.icon,
                            post: rdef2.name + " x" + numberWithCommas(company.resourceGen.output[j].amount),
                            color: "#4CAF50"
                        });
                    }
                }
            }

            if(!canBuild) {
                continue;
            }

            businessListItems.push({
                "title":        company.name,
                "title-color":  null,    
                "description":  useDesc,
                "status":       preText,
                "status-color": preColor,
                "icon":         "images/game/" + company.image,
                "overflow":     overflowItems,
                "tag":          useTag
            });

        }
    }

    function handleTimer(start) {
        if(start && currentGame) {
            gameLoop = setInterval(inGameUpdate,currentGame.tickLength);
        } else {
            clearInterval(gameLoop);
        }
    }

    function loadGame() {    
        webAppCore.openListing(onListingBack);
    }

    function onListingBack(listing) {

        let saveExists = false;

        for(let i = 0; i < listing.length; i++) {
            if(listing[i] == "mbe_save.mbe") {
                saveExists = true;
            }
        }
        
        if (saveExists) {
            webAppCore.openFile("mbe_save.mbe",onGameFileOpened);
        } else {
            if (!currentGame || currentGame.cash < 100) {
                defaultOutVars();
                handleTimer(true);

                webAppCore.clearSplash();
            }
            
        }

    }

    function onGameFileOpened(data) {

        if(!data) {
            if(!currentGame) {
                defaultOutVars();
                handleTimer(true);
            }

            webAppCore.clearSplash();
            
            populateBusinessListing();

            return;
        }
        
        let savedRawGameData;
        let prop;
        
        try { 
            savedRawGameData = atob(data);
        } catch(err) {

            webAppCore.logEvent("error","loadgame1",err);
            
            savedRawGameData = data;
        }

        
        
        try {
            let tmpGame = JSON.parse(savedRawGameData);

            if(currentGame) {
                if(currentGame.cash > tmpGame.cash) {
                    webAppCore.clearSplash();
                    populateBusinessListing();
                    return;
                }
            }

            document.getElementById("revenueReportBody").innerHTML = "";

            currentGame = new GameData();
            
            for (prop in tmpGame) {
                currentGame[prop] = tmpGame[prop];
            }

            populateBusinessListing();
        } catch(err) {

            webAppCore.logEvent("error","loadgame2",err);
            
            defaultOutVars();
            handleTimer(true);

            webAppCore.clearSplash();
            populateBusinessListing();
            return;
        }
        
        if (currentGame.businesses.length > 0) {
            let rawObjects = [];

            for (let i = 0; i < currentGame.businesses.length; i++) {

                let spr = new Business(currentGame.businesses[i].name,currentGame.businesses[i].image,currentGame.businesses[i].profit,currentGame.businesses[i].rate,currentGame.businesses[i].cost,currentGame.businesses[i].baseWorkers);

                for (prop in currentGame.businesses[i]) {
                    spr[prop] = currentGame.businesses[i][prop];
                }

                if(spr.businessId == 0) {
                    spr.businessId = currentGame.nextBusinessId;
                    currentGame.nextBusinessId++;
                }

                rawObjects[rawObjects.length] = spr;

            }
            
            currentGame.businesses = rawObjects;
        }
        
        playCatchup();


        webAppCore.clearSplash();
        populateBusinessListing();

        if(currentGame.businesses.length > 0) {
            if(allowAudio) {
                if(currentSong == null) {
                    playNextSong();
                }

            }
        }

    }

    function onPause() {

        handleTimer(false);

        //saveTime();
        gamePaused = true;
    }

    function onResume() {
        if(gamePaused) {

            playCatchup();
            
            webAppCore.clearSplash();
        }
    }
    
    function onMenuOptionSelected(name,tag) {

        if(tag == "externallink") {
            if(!ManagementGame.externalLink) {
                return;
            }

            let useURL = ManagementGame.externalLink.default;

            let plat = webAppCore.getPlatform();

            if(plat.platform == "android" && plat.market == "gplay") {
                if(ManagementGame.externalLink.gplay) {
                    useURL = ManagementGame.externalLink.gplay;
                }
            }

            if(plat.platform == "android" && plat.market == "amazon") {
                if(ManagementGame.externalLink.amazon) {
                    useURL = ManagementGame.externalLink.amazon;
                }
            }

            if(plat.platform == "windows") {
                if(ManagementGame.externalLink.windows) {
                    useURL = ManagementGame.externalLink.windows;
                }
            }

            if(plat.platform == "ios") {
                if(ManagementGame.externalLink.itunes) {
                    useURL = ManagementGame.externalLink.itunes;
                }
            }

            webAppCore.externalURL(useURL);
        }

        if(tag == "tech") {
            showTechWindow();
        }

        if(tag == "resources") {
            showResources();
        }

        if(tag == "soundoff") {
            allowAudio = false;
            gooseUpMenu();
            webAppCore.stopMidi();

            webAppCore.storeVar("mbe-allow-audio","0");
        }

        if(tag == "soundon") {
            allowAudio = true;
            gooseUpMenu();

            

            if(currentGame.businesses.length > 0) {
                playNextSong();
            }

            webAppCore.storeVar("mbe-allow-audio","1");
        }

        if(allowAudio) {
            playUiClick();
        }

        if(tag == "new") {
            newGame();
        }
        
        if(tag == "payroll") {
            showAccountingScreen();
        }

        if(tag == "legal") {
            showLegalScreen();
        }
        
        if(tag == "employees") {
            showHR();
        }
        
        if(tag == "rename") {
            renameSelected();
        }

        if(tag == "community") {
            webAppCore.externalURL(ManagementGame.meta.communityUrl);
        }
        
        if(tag == "sell") {
            if(currentGame.businesses.length == 0) {
                webAppCore.showAlertWithTitle("You must start a new business first before you can name it!","No business!");
                return;
            }
        
            let ourIndex = currentGame.selectedBusinessIndex;
        
            if(isNaN(ourIndex) || ourIndex >= currentGame.businesses.length) {
                ourIndex = 0;
            }
            
            currentGame.selectedBusinessIndex = ourIndex;
            
            sellBusiness(currentGame.businesses[ourIndex]);
        }
    }

    function playUiClick() {
        if(allowAudio) {
            webAppCore.playSoundEffect("select");
        }
    }

    function onPadDown(id,button) {

        if(!currentGame) {
            return;
        }

        if(button == "right") {
            playUiClick();
            nextBusiness();
        }

        if(button == "left") {
            playUiClick();
            previousBusiness();
        }

        if(button == "select" || button == "start" || button == "up" || button == "down" || button == "b") {
            playUiClick();
            webAppCore.toggleMenu();
        }

        if(button == "a") {
            startRunCycle();
        }

        if(button == "b") {
            document.getElementById("revenueReport").style.display = "none";
            webAppCore.handleBackFunction(null);
        }

        if(button == "lb") {
            stackUpBusiness(currentGame.businesses[resumeAndGetCurrentBusinessIndex()]);
        }

        if(button == "rb" || button == "x") {
            toggleFinancial();
            playUiClick();
        }

        if(button == "lt") {
            levelUpBusiness(currentGame.businesses[resumeAndGetCurrentBusinessIndex()]);
        }

        if(button == "rt") {
            addManager(currentGame.businesses[resumeAndGetCurrentBusinessIndex()]);
        }

        if(button == "y") {
            toggleNewBusinessListing();
            playUiClick();
        }

    }

    // id,button
    function onPadUp() {
        // nothing right now
    }

    function setupHandlers() {
        
        singleCardHolder = document.getElementById("singleCardHolder");
        allCardsContainer = document.getElementById("allCardsContainer");

        document.getElementById("wrapper").addEventListener("click",function() {
            document.getElementById("revenueReport").style.display = "none";
            webAppCore.handleBackFunction(null);
        },false);
        
        document.getElementById("menuToggle").addEventListener("click",function(e) {
            e.stopPropagation();
            toggleMenu();
            playUiClick();
            
        },false);
        
        document.getElementById("timeReadout").addEventListener("click",function(e) {
            e.stopPropagation();
            showEmployees();
            playUiClick();
        },false);

        document.getElementById("btnNewBusiness").addEventListener("click",function(e) {
            e.stopPropagation();
            toggleNewBusinessListing();
            playUiClick();
        },false);

        document.getElementById("btnFinancial").addEventListener("click",function(e) {
            e.stopPropagation();
            toggleFinancial();
            playUiClick();
        },false);
        
        document.getElementById("moneyHolder").addEventListener("click",function(e) {
            e.stopPropagation();

            if(ManagementGame.resources && ManagementGame.resources.length > 0) {
                showResources();
            } else {
                toggleFinancial();
            }

            
            playUiClick();
        },false);

        document.getElementById("btnNext").addEventListener("click",function(e) {
            e.stopPropagation();
            nextBusiness();
            playUiClick();
        },false);

        document.getElementById("btnPrevious").addEventListener("click",function(e) {
            e.stopPropagation();
            previousBusiness();
            playUiClick();
        },false);
        
        
        document.getElementById("businessCardImage").addEventListener("click",function(e) {
            e.stopPropagation();

            var revenueReport = document.getElementById("revenueReport");
            if (revenueReport.style.display == "block") {
                toggleFinancial();
            }

            startRunCycle();
        },false);
        
        document.getElementById("btnRestart").addEventListener("click",function(e) {
            e.stopPropagation();

            var revenueReport = document.getElementById("revenueReport");
            if (revenueReport.style.display == "block") {
                toggleFinancial();
            }

            startRunCycle();
        },false);
        
        document.getElementById("levelBoostArea").addEventListener("click",function(e) {
            e.stopPropagation();
            
            var ourIndex = resumeAndGetCurrentBusinessIndex();
            
            levelUpBusiness(currentGame.businesses[ourIndex]);

        },false);

        document.getElementById("stackBoostArea").addEventListener("click",function(e) {
            e.stopPropagation();
            
            var ourIndex = resumeAndGetCurrentBusinessIndex();
            
            stackUpBusiness(currentGame.businesses[ourIndex]);

        },false);
        
        document.getElementById("managerBoostArea").addEventListener("click",function(e) {
            e.stopPropagation();

            var ourIndex = resumeAndGetCurrentBusinessIndex();

            addManager(currentGame.businesses[ourIndex]);

        },false);
        
        document.getElementById("businesslevelstat").addEventListener("click",function(e) {
            e.stopPropagation();
            levelInfo();
        },false);
        
        document.getElementById("businessCardName").addEventListener("click",function(e) {
            e.stopPropagation();
            renameSelected();
            playUiClick();
        },false);
       
        document.getElementById("businessManagerStat").addEventListener("click",function(e) {
            e.stopPropagation();

            var ourIndex = resumeAndGetCurrentBusinessIndex();
            managementInfo(currentGame.businesses[ourIndex]);
        },false);
        
        document.getElementById("businessWorkerStat").addEventListener("click",function(e) {
            e.stopPropagation();

            var ourIndex = resumeAndGetCurrentBusinessIndex();
            workerInfo(currentGame.businesses[ourIndex].workers);
        },false);

        document.getElementById("businessStorageStat").addEventListener("click",function(e) {
            e.stopPropagation();

            showResources();
        },false);
    }
    
    function resumeAndGetCurrentBusinessIndex() {
        if(gamePaused) {
            onResume();
        }
        
        if(currentGame.businesses.length == 0) {
            return null;
        }
        
        let ourIndex = currentGame.selectedBusinessIndex;
        
        if(isNaN(ourIndex) || ourIndex >= currentGame.businesses.length) {
            ourIndex = 0;
        }
        
        currentGame.selectedBusinessIndex = ourIndex;
        
        return ourIndex;
    }

    function toggleNewBusinessListing() {

        let revenueReport = document.getElementById("revenueReport");
        if(revenueReport.style.display == "block") {
            revenueReport.style.display = "none";
            financialReportOpen = false;
        }

        populateBusinessListing();

        let showItems = [];
        let i;

        let availableTechs = 0;

        // check if there are techs
        for(i = 0; i < ManagementGame.technologies.length; i++) {

            if(currentGame.unlockedTechs.indexOf(ManagementGame.technologies[i].name) > -1) {
                continue;
            }

            availableTechs++;
        }

        if(availableTechs > 0) {
            showItems.push({
                "title":    "New Technology Available",
                "description":  "Unlock more businesses by researching the tech tree.",
                "icon": "images/ui/ind-research.png",
                "tag": "techtree"
            });
        }


        for(i = 0; i < businessListItems.length; i++) {
            if(businessListItems[i].tag != "reqpremium") {
                if(businessListItems[i].tag.cost > currentGame.cash) {
                    businessListItems[i]["status-color"] = "#F44336";
                } else {
                    businessListItems[i]["status-color"] = null;
                }
            }

            showItems.push(businessListItems[i]);
        }

        webAppCore.presentNativeList("New Business",showItems,openBusiness,"Nevermind");

    }
    
    function workerInfo(num) {
        let totalSalary = num * currentGame.workerSalary;
        totalSalary = totalSalary / 12;

        webAppCore.showAlertWithTitle("The number of workers it takes to run this business.  Each worker is paid the company-wide annual salary of " + numberToCash(currentGame.workerSalary) + " (at this business, that works out to " + numberToCash(totalSalary) + " per month).","Workers");
    }

    function levelInfo() {
        webAppCore.showAlertWithTitle("Your business level determines how much income this business can generate.  The higher the level, the higher the income.","Business Level");
    }

    var tmpBusinessToFire;

    function managementInfo(business) {
        let totalSalary = business.manager * currentGame.managerSalary;
        totalSalary = totalSalary / 12;
        
        let baseInfo = "The Management Level determines how automated this business is.  With no management, employees will do the minimal amount of work to get paid.  Each manager is paid the company-wide annual management salary of " + numberToCash(currentGame.managerSalary);
        
        if(business.manager > 0) {
            tmpBusinessToFire = business;
            baseInfo += " (at this business, that works out to " + numberToCash(totalSalary) + " per month).  Would you like to fire one of your managers?";
            webAppCore.showChoice(baseInfo,"Fire a Manager","Ok",fireAManger,placeholder);
        } else {
            baseInfo += ".";
            webAppCore.showAlertWithTitle(baseInfo,"Management Level");
        }
        
    }
    
    function fireAManger() {
        if(tmpBusinessToFire && tmpBusinessToFire != null) {
            subtractManager(tmpBusinessToFire);
        }
    }

    function setupResourceDisplays() {

        document.getElementById("resourcesHolder").innerHTML = "";

        for(let i = 0; i < ManagementGame.resources.length; i++) {
            let res = ManagementGame.resources[i];

            let holderId = "res-disp-" + res.name + "-holder";
            let spanId = "res-disp-" + res.name + "-display";

            let holder = document.createElement("div");
            holder.id = holderId;
            holder.className = "resourceDisplayHolder";
            holder.title = res.name;

            let resIcon = document.createElement("img");
            resIcon.src = "images/resources/" + res.icon;
            resIcon.className = "resourceDisplayItemIcon";
            holder.appendChild(resIcon);

            let resSpan = document.createElement("span");
            resSpan.className = "resourceDisplayItemSpan";
            resSpan.id = spanId;
            holder.appendChild(resSpan);

            document.getElementById("resourcesHolder").appendChild(holder);
        }
    }

    function updateDisplays() {

        let i,l;

        if(window.innerWidth >= 500) {
            for(i = 0; i < ManagementGame.resources.length; i++) {
                let res = ManagementGame.resources[i];
                let useDisp = "block";
                let holderId = "res-disp-" + res.name + "-holder";
                let spanId = "res-disp-" + res.name + "-display";

                if(!currentGame.resources[res.name]) {
                    if(document.getElementById(holderId).style.display != "none") {
                        document.getElementById(holderId).style.display = "none";
                    }
                    continue;
                }

                let gameRes = currentGame.resources[res.name];

                
                if(!gameRes.isUnlocked) {
                    useDisp = "none";
                }

                let useAmt = numberWithCommas(gameRes.inv);

                if(gameRes.inv == 0) {
                    useAmt = "0";
                }

                if(document.getElementById(spanId).innerHTML != useAmt) {
                    document.getElementById(spanId).innerHTML = useAmt;
                }

                if(document.getElementById(holderId).style.display != useDisp) {
                    document.getElementById(holderId).style.display = useDisp;
                }
            }
        }
        

        let popReadout = document.getElementById("population");
        let rawCadhRsadou = numberToCash(currentGame.cash);

        if(popReadout.innerHTML != rawCadhRsadou) {
            popReadout.innerHTML = rawCadhRsadou;
        }
        
        if(currentGame.cash <= 0) {
            popReadout.style.color = "#B71C1C";
        } else {
            popReadout.style.color = null;
        }
        
        let hourDisplay = currentGame.hour;
        
        if(currentGame.hour < 0) {
            hourDisplay = "0" + currentGame.hour;
        }
        
        hourDisplay = hourDisplay + ":00";

        let timeReadoutRaw = null;
        
        if(window.innerWidth > 500) {
            timeReadoutRaw = "Year: " + (currentGame.year + 1) + " Month: " + (currentGame.month + 1) + " Day: " + (currentGame.day + 1) + " " + hourDisplay;
        } else {
            timeReadoutRaw = "Y" + (currentGame.year + 1) + " M" + (currentGame.month + 1) + " D" + (currentGame.day + 1) + " " + hourDisplay;
        }

        let levelButtonDisplay = null;
        let managerButtonDisplay = null;
        let levelIndicatorDisplay = null;
        let managerIndicatorDisplay = null;
        let workerIndicatorDisplay = null;
        let meterDisplay = null;
        let storageDisplay = null;
        let storageText = null;
        let profitDisplay = null;

        let waitingOnDisplay = null;
        let waitingOnContent = "";

        let storageMeterDisplay = null;
        let storageMeterWidth = "0%";
        let storageMeterColor = null;

        if(currentGame.currentStorage > 0) {
            let percent =  Math.round((currentGame.currentStorage / currentGame.storageCap) * 100);

            storageMeterWidth = percent + "%";

            if(percent > 90) {
                storageMeterColor = "#d32f2f";
            }
        } else {
            storageMeterDisplay = "none";
        }

        if(document.getElementById("storageMeterHolder").style.display != storageMeterDisplay) {
            document.getElementById("storageMeterHolder").style.display = storageMeterDisplay;
        }

        if(document.getElementById("storageMeter").style.backgroundColor != storageMeterColor) {
            document.getElementById("storageMeter").style.backgroundColor = storageMeterColor;
        }

        if(document.getElementById("storageMeter").style.width != storageMeterWidth) {
            document.getElementById("storageMeter").style.width = storageMeterWidth;
        }

        if(document.getElementById("timeReadout").innerHTML != timeReadoutRaw) {
            document.getElementById("timeReadout").innerHTML = timeReadoutRaw;
        }
        
        if(currentGame.day >= 29) {
            document.getElementById("timeReadout").style.color = "#F44336";
        } else {
            document.getElementById("timeReadout").style.color = null;
        }
        
        if(currentGame.businesses.length == 0) {
            document.getElementById("noBusinessWarning").style.display = "block";
            singleCardHolder.style.display = "none";
        } else {
            document.getElementById("noBusinessWarning").style.display = "none";
            singleCardHolder.style.display = "block";
            
            
            let ourIndex = currentGame.selectedBusinessIndex;
            
            if(isNaN(ourIndex) || ourIndex >= currentGame.businesses.length) {
                ourIndex = 0;
            }

            let theShowingBusiness = currentGame.businesses[ourIndex];

            let stackUpPrice = getStackPrice(theShowingBusiness);
            let stackColor = "rgba(0,0,180,.5)";
            let stackPriceReadout = numberToCash(stackUpPrice);
            let stackVisibility = null;

            let bdef = getBusinessDefByName(theShowingBusiness.name);

            if(bdef.oneOffJob) {
                stackVisibility = "none";
            }

            if(stackUpPrice > currentGame.cash) {
                stackColor = "rgba(180,0,0,.5)";
            }

            if(document.getElementById("addStackCost").innerHTML != stackPriceReadout) {
                document.getElementById("addStackCost").innerHTML = stackPriceReadout;
            }

            if(document.getElementById("stackBoostArea").style.backgroundColor != stackColor) {
                document.getElementById("stackBoostArea").style.backgroundColor = stackColor;
            }

            if(document.getElementById("stackBoostArea").style.display != stackVisibility) {
                document.getElementById("stackBoostArea").style.display = stackVisibility;
            }

            let storageCount = 0;

            if(bdef) {
                if(bdef.profit == 0) {
                    profitDisplay = "none";
                    levelButtonDisplay = "none";
                    levelIndicatorDisplay = "none";

                    if(!bdef.resourceGen || (bdef.resourceGen && bdef.resourceGen.output.length == 0)) {
                        managerButtonDisplay = "none";
                        managerIndicatorDisplay = "none";
                    }

                    if(!bdef.resourceGen || (bdef.resourceGen && bdef.resourceGen.output.length == 0 && bdef.resourceGen.input.length == 0)) {
                        meterDisplay = "none";
                    }
                }
                
                if(bdef.workers == 0) {
                    workerIndicatorDisplay = "none";
                }

                if(bdef.runsWithoutManagement && bdef.workers == 0) {
                    managerButtonDisplay = "none";
                    managerIndicatorDisplay = "none";
                }

                if(bdef.storageMod && bdef.storageMod != 0) {
                    storageCount = parseInt(bdef.storageMod * theShowingBusiness.stack);
                }

                let waitingOnResources = [];

                if(ManagementGame.resources) {
                    for(i = 0; i < ManagementGame.resources.length; i++) {
                        let resDisp = "none";
                        let resid = "meter-readout-resource-" + ManagementGame.resources[i].name;

                        if(!document.getElementById(resid)) {
                            continue;
                        }

                        if(bdef.resourceGen && bdef.resourceGen.output.length > 0) {
                            for(l = 0; l < bdef.resourceGen.output.length; l++) {
                                if(bdef.resourceGen.output[l].resource == ManagementGame.resources[i].name) {
                                    resDisp = null;
                                    break;
                                }
                            }
                        }

                        if(!theShowingBusiness.isRunning) {

                            if(bdef.resourceGen && bdef.resourceGen.input.length > 0) {

                                for(l = 0; l < bdef.resourceGen.input.length; l++) {
                                    if(bdef.resourceGen.input[l].resource == ManagementGame.resources[i].name) {

                                        if(bdef.resourceGen.input[l].amount > currentGame.resources[ManagementGame.resources[i].name].inv) {

                                            let rDef = getResourceDefByName(ManagementGame.resources[i].name);

                                            if(rDef) {
                                                waitingOnResources.push("<img src=\"images/resources/" + rDef.icon + "\" />");
                                            }
                                            
                                        }
                                    }
                                }
                            }
                        }

                        if(document.getElementById(resid).style.display != resDisp) {
                            document.getElementById(resid).style.display = resDisp;
                        }
                    }

                    if(waitingOnResources.length > 0) {
                        waitingOnDisplay = null;
                        waitingOnContent = "Waiting on: ";
        
                        for(l = 0; l < waitingOnResources.length; l++) {
                            waitingOnContent += waitingOnResources[l];
                        }
                    } else {
                        waitingOnDisplay = "none";
                        waitingOnContent = "";
                    }
                }
            }

            

            if(document.getElementById("businessCardWaitingOnHolder").style.display != waitingOnDisplay) {
                document.getElementById("businessCardWaitingOnHolder").style.display = waitingOnDisplay;
            }

            if(lastWaitingOnContent != waitingOnContent) {
                lastWaitingOnContent = waitingOnContent;
                document.getElementById("businessCardWaitingOnHolder").innerHTML = waitingOnContent;
            }

            if(storageCount == 0) {
                storageDisplay = "none";
                storageText = "";
            } else {
                storageDisplay = null;
                storageText = storageCount.toString();
            }

            let maxMgr = 5;

            if(theShowingBusiness.baseWorkers < 5) {
                maxMgr = theShowingBusiness.baseWorkers;
            }

            if(maxMgr < 1) {
                if(bdef.runsWithoutManagement) {
                    maxMgr = 0;
                } else {
                    maxMgr = 1;
                }
                
            }

            if(theShowingBusiness.level >= 5) {
                levelButtonDisplay = "none";
            }

            if(theShowingBusiness.manager >= maxMgr) {
                managerButtonDisplay = "none";
            }

            if(document.getElementById("businessCardMeterReadoutCash").style.display != profitDisplay) {
                document.getElementById("businessCardMeterReadoutCash").style.display = profitDisplay;
            }

            if(document.getElementById("businessCardStorageDisplay").innerHTML != storageText) {
                document.getElementById("businessCardStorageDisplay").innerHTML = storageText;
            }

            if(document.getElementById("businessStorageStat").style.display != storageDisplay) {
                document.getElementById("businessStorageStat").style.display = storageDisplay;
            }

            if(document.getElementById("businessCardMeterHolder").style.display != meterDisplay) {
                document.getElementById("businessCardMeterHolder").style.display = meterDisplay;
            }

            if(document.getElementById("levelBoostArea").style.display != levelButtonDisplay) {
                document.getElementById("levelBoostArea").style.display = levelButtonDisplay;
            }

            if(document.getElementById("businesslevelstat").style.display != levelIndicatorDisplay) {
                document.getElementById("businesslevelstat").style.display = levelIndicatorDisplay;
            }

            if(document.getElementById("managerBoostArea").style.display != managerButtonDisplay) {
                document.getElementById("managerBoostArea").style.display = managerButtonDisplay;
            }

            if(document.getElementById("businessManagerStat").style.display != managerIndicatorDisplay) {
                document.getElementById("businessManagerStat").style.display = managerIndicatorDisplay;
            }

            if(document.getElementById("businessWorkerStat").style.display != workerIndicatorDisplay) {
                document.getElementById("businessWorkerStat").style.display = workerIndicatorDisplay;
            }

            currentGame.selectedBusinessIndex = ourIndex;

            let nameReadout = theShowingBusiness.customName;

            if(theShowingBusiness.stack && theShowingBusiness.stack > 1) {
                nameReadout += " (x" + theShowingBusiness.stack + ")";
            }

            let updatePercentage = businessUpdatePercent(theShowingBusiness);

            document.getElementById("businessCardImage").style.backgroundImage = "url('images/game/" + theShowingBusiness.image + "')";

            if(document.getElementById("businessCardName").innerHTML != nameReadout) {
                document.getElementById("businessCardName").innerHTML = nameReadout;
            }

            if(document.getElementById("businessCardLevelDisplay").innerHTML != theShowingBusiness.level) {
                document.getElementById("businessCardLevelDisplay").innerHTML = theShowingBusiness.level;
            }

            if(document.getElementById("businessCardManagerDisplay").innerHTML != theShowingBusiness.manager * theShowingBusiness.stack) {
                document.getElementById("businessCardManagerDisplay").innerHTML = theShowingBusiness.manager * theShowingBusiness.stack;

                if(theShowingBusiness.manager == 0) {
                    if(document.getElementById("businessCardManagerDisplay").style.color != "#F44336") {
                        document.getElementById("businessCardManagerDisplay").style.color = "#F44336";
                    }
                } else {
                    if(document.getElementById("businessCardManagerDisplay").style.color != null) {
                        document.getElementById("businessCardManagerDisplay").style.color = null;
                    }
                }
            }

            if(document.getElementById("businessCardWorkerDisplay").innerHTML != theShowingBusiness.workers * theShowingBusiness.stack) {
                document.getElementById("businessCardWorkerDisplay").innerHTML = theShowingBusiness.workers * theShowingBusiness.stack;
            }


            if(document.getElementById("businessCardMeter").style.width != updatePercentage + "%") {
                document.getElementById("businessCardMeter").style.width = updatePercentage + "%";
            }

            let meterReadoutNum = numberToCash(getModifiedProfit(theShowingBusiness));

            if(document.getElementById("businessCardMeterReadoutCash").innerHTML != meterReadoutNum) {
                document.getElementById("businessCardMeterReadoutCash").innerHTML = meterReadoutNum;
            }

            let luc = numberToCash(theShowingBusiness.levelUpCost * theShowingBusiness.stack);

            let levelUpHeaderText = "Level Up";

            if(theShowingBusiness.level >= 5) {
                levelUpHeaderText = "Max";
                luc = "-";
            }

            if(document.getElementById("businessCardLevelCost").innerHTML != luc) {
                document.getElementById("businessCardLevelCost").innerHTML = luc;
            }

            let muc = numberToCash(theShowingBusiness.managerUpCost * theShowingBusiness.stack);

            if(theShowingBusiness.manager >= maxMgr) {
                muc = "-";
            }
            
            if(document.getElementById("businessCardManagerCost").innerHTML != muc) {
                document.getElementById("businessCardManagerCost").innerHTML = muc;
            }
            
            let rps = getBusinessRevenuePerDay(theShowingBusiness);
            let dailyEstReadout = "Est. $/Day: " + numberToCash(rps);
            let dailyEstColor = null;

            if(rps > 0) {
                dailyEstColor = null;
            } else {
                dailyEstColor = "#B71C1C";
            }

            if(document.getElementById("businessCardRevenue").innerHTML != dailyEstReadout) {
                document.getElementById("businessCardRevenue").innerHTML = dailyEstReadout;
            }

            if(document.getElementById("businessCardRevenue").style.color != dailyEstColor) {
                document.getElementById("businessCardRevenue").style.color = dailyEstColor;
            }
            
            

            if(document.getElementById("levelUpHeader").innerHTML != levelUpHeaderText) {
                document.getElementById("levelUpHeader").innerHTML = levelUpHeaderText;
            }
            
            if(currentGame.cash < (theShowingBusiness.levelUpCost * theShowingBusiness.stack) || theShowingBusiness.level >= 5) {
                if(document.getElementById("levelBoostArea").style.backgroundColor != "rgba(180,0,0,.5)") {
                    document.getElementById("levelBoostArea").style.backgroundColor = "rgba(180,0,0,.5)";
                }
            } else {
                if(document.getElementById("levelBoostArea").style.backgroundColor != "rgba(0,0,180,.5)") {
                    document.getElementById("levelBoostArea").style.backgroundColor = "rgba(0,0,180,.5)";
                }
                
            }

            let addManagerHeaderText = "Add Manager";

            if(theShowingBusiness.manager >= maxMgr) {
                addManagerHeaderText = "Max";
            }

            if(document.getElementById("addManagerHeader").innerHTML != addManagerHeaderText) {
                document.getElementById("addManagerHeader").innerHTML = addManagerHeaderText;
            }
            
            if(currentGame.cash < (theShowingBusiness.managerUpCost * theShowingBusiness.stack) || theShowingBusiness.manager >= maxMgr) {
                if(document.getElementById("managerBoostArea").style.backgroundColor != "rgba(180,0,0,.5)") {
                    document.getElementById("managerBoostArea").style.backgroundColor = "rgba(180,0,0,.5)";
                }
            } else {
                if(document.getElementById("managerBoostArea").style.backgroundColor != "rgba(0,0,180,.5)") {
                    document.getElementById("managerBoostArea").style.backgroundColor = "rgba(0,0,180,.5)";
                }
            }

            updateListAllDisplay();

        }
        
        if(currentGame.selectedBusinessIndex > 0) {
            if(document.getElementById("btnPrevious").src != "images/ui/previous.png") {
                document.getElementById("btnPrevious").src = "images/ui/previous.png";
            }
        } else {
            if(document.getElementById("btnPrevious").src != "images/ui/previous-no.png") {
                document.getElementById("btnPrevious").src = "images/ui/previous-no.png";
            }
        }
        
        if(currentGame.selectedBusinessIndex < currentGame.businesses.length - 1) {
            if(document.getElementById("btnNext").src != "images/ui/next.png") {
                document.getElementById("btnNext").src = "images/ui/next.png";
            }
        } else {
            if(document.getElementById("btnNext").src != "images/ui/next-no.png") {
                document.getElementById("btnNext").src = "images/ui/next-no.png";
            }
        }
    }
    

    function startRunCycle() {
        
        if(gamePaused) {
            onResume();
        }
        
        if(currentGame.businesses.length == 0) {
            return;
        }
        
        let ourIndex = currentGame.selectedBusinessIndex;
        
        if(isNaN(ourIndex) || ourIndex >= currentGame.businesses.length) {
            ourIndex = 0;
        }
        
        currentGame.selectedBusinessIndex = ourIndex;
        
        runBusiness(currentGame.businesses[ourIndex]);

        if(!currentGame.businesses[ourIndex].isRunning) {
            webAppCore.showToast("Waiting on Resources!");
        }
    }

    

    function nextBusiness() {
        if(gamePaused) {
            onResume();
        }

        if(!currentGame) {
            return;
        }
        
        if(currentGame.businesses.length == 0) {
            return;
        }
        
        let ourIndex = currentGame.selectedBusinessIndex;
        
        if(isNaN(ourIndex) || ourIndex >= currentGame.businesses.length) {
            ourIndex = 0;
        }
        
        if((ourIndex + 2) > currentGame.businesses.length) {
            return;
        }
        
        ourIndex++;
        
        currentGame.selectedBusinessIndex = ourIndex;

        updateDisplays();
    }

    function previousBusiness() {
        if(gamePaused) {
            onResume();
        }
        
        if(!currentGame) {
            return;
        }

        if(currentGame.businesses.length == 0) {
            return;
        }
        
        let ourIndex = currentGame.selectedBusinessIndex;
        
        if(isNaN(ourIndex) || ourIndex >= currentGame.businesses.length) {
            ourIndex = 0;
        }
        
        if(ourIndex == 0) {
            return;
        }
        
        ourIndex--;
        
        currentGame.selectedBusinessIndex = ourIndex;

        updateDisplays();
    }

    function stackUpBusiness(business) {
        if(gamePaused) {
            onResume();
        }

        if(!business) {
            return;
        }

        if(currentGame.jailDaysRemaining > 0) {
            webAppCore.showAlertWithTitle("You cannot open a business while you are in jail!  You still have " + currentGame.jailDaysRemaining + " days remaining.","In Jail");
            return;
        }

        let stackCost = getStackPrice(business);

        if(stackCost > currentGame.cash) {
            webAppCore.showAlertWithTitle("You do not have enough money to open this business!","Insufficient Funds");
            return;
        }

        if(allowAudio) {
            webAppCore.playSoundEffect("chaching");
        }

        modifyCashAmount(-stackCost,false);

        business.stack++;

        saveTime();
    }

    function levelUpBusiness(business) {
        if(gamePaused) {
            onResume();
        }

        if(!business) {
            return;
        }

        if(currentGame.jailDaysRemaining > 0) {
            webAppCore.showAlertWithTitle("You cannot level up a business while you are in jail!  You still have " + currentGame.jailDaysRemaining + " days remaining.","In Jail");
            return;
        }

        if(business.level >= 5) {
            webAppCore.showAlertWithTitle("You can not level this business up any further.","Max Level");
            return;
        }

        let totCost = business.levelUpCost * business.stack;

        if(totCost > currentGame.cash) {
            webAppCore.showAlertWithTitle("You do not have enough money to level up this business!","Insufficient Funds");
            return;
        }

        if(allowAudio) {
            webAppCore.playSoundEffect("chaching");
        }

        

        modifyCashAmount(-totCost,false);

        business.level++;
        business.levelUpCost = business.levelUpCost + Math.ceil(business.levelUpCost * 0.45);
        
        business.profit = business.profit + (business.profit * business.levelUpBenefit);
        
        
        business.levelUpBenefit = business.levelUpBenefit * 0.9;

        if(business.sellValue && business.sellValue > 0) {
            business.sellValue += Math.ceil(business.sellValue * 0.05);
        }
        
        saveTime();
    }

    function modifyResourceAmount(res,amount) {
        let cleanAmount = parseInt(amount);

        

        let currentStorageUsed = 0;
        let sres;

        for(sres in currentGame.resources) {
            currentStorageUsed += currentGame.resources[sres].inv;
        }

        if(currentStorageUsed > currentGame.storageCap) {
            currentStorageUsed = currentGame.storageCap;
        }

        let storageAvailable = currentGame.storageCap - currentStorageUsed;

        if(storageAvailable < 0) {
            storageAvailable = 0;
        }

        if(cleanAmount > storageAvailable) {
            cleanAmount = storageAvailable;
        }

        let resDef = getResourceDefByName(res);

        if(cleanAmount > 0 && !currentGame.firstResourceDepositDone) {
            currentGame.firstResourceDepositDone = true;
            webAppCore.showDialog({
                title: "Resources",
                message: "You have just collected " + res + ", your first resource!  Resources are stored and used to operate other businesses and make profit.  Check and manage your resources and storage from the Resources dialog found in the main game menu.",
                icon: "images/resources/" + resDef.icon,
                buttons: [
                    {
                        text: "Show Resources",
                        func: showResources,
                        color: "#4CAF50"
                    },
                    {
                        text: "Dismiss",
                        func: placeholder
                    }
                ]
            });
        }

        if(!currentGame.resources[res]) {
            currentGame.resources[res] = new Resource(resDef);
        }

        currentGame.resources[res].inv += cleanAmount;

        if(currentGame.resources[res].inv < 0) {
            currentGame.resources[res].inv = 0;
        }

        if(currentGame.resources[res].inv > currentGame.storageCap) {
            currentGame.resources[res].inv = currentGame.storageCap;
        }

        if(!currentGame.resources[res].isUnlocked) {
            currentGame.resources[res].isUnlocked = true;

            webAppCore.showToast({
                message: "Resource Unlocked: " + res,
                image: "images/resources/" + resDef.icon,
                action: {
                    title: "Show",
                    func: showResources
                }
            });

            populateBusinessListing();
        }

        // update the new total storage
        currentStorageUsed = 0;

        for(sres in currentGame.resources) {
            currentStorageUsed += currentGame.resources[sres].inv;
        }

        currentGame.currentStorage = currentStorageUsed;
    }

    function modifyCashAmount(amount,illegal) {
        
        if(isNaN(amount)) {
            return;
        }
        
        if(isNaN(currentGame.cash)) {
            currentGame.cash = 0;
        }
        
        amount = parseInt(amount);
        
        //console.log(amount);

        currentGame.cash += amount;

        if(amount > 0) {
            if(illegal) {

                if(!currentGame.accounting.illegal) {
                    currentGame.accounting.illegal = {
                        "quarter": 0,
                        "allTime": 0
                    };
                }

                currentGame.accounting.illegal.quarter += amount;
                currentGame.accounting.illegal.allTime += amount;
            } else {
                currentGame.accounting.revenue.quarter += amount;
                currentGame.accounting.revenue.allTime += amount;
            }
            
        } else {
            currentGame.accounting.expense.quarter -= amount;
            currentGame.accounting.expense.allTime -= amount;
        }
    }
    
    function subtractManager(business) {
        if(gamePaused) {
            onResume();
        }
        
        if(business.manager == 0) {
            webAppCore.showAlertWithTitle("This business has no managers to fire!","No Managers");
            return;
        }
        
        business.manager--;

        saveTime();
        
        webAppCore.showAlertWithTitle("One manager has been escorted off the premises.","Manager Fired");
    }

    function addManager(business) {
        if(gamePaused) {
            onResume();
        }

        if(!business) {
            return;
        }

        let maxMgr = 5;

        if(business.baseWorkers < 5) {
            maxMgr = business.baseWorkers;
        }

        if(maxMgr < 1) {
            maxMgr = 1;
        }

        if(business.manager >= maxMgr) {
            webAppCore.showAlertWithTitle("This business already has the maximum number of hired managers!","Max Managers");
            return;
        }

        if(business.managerUpCost > currentGame.cash) {
            webAppCore.showAlertWithTitle("You do not have enough money to hire a new manager!","Insufficient Funds");
            return;
        }

        if(currentGame.jailDaysRemaining > 0) {
            webAppCore.showAlertWithTitle("You cannot hire managers while you are in jail!  You still have " + currentGame.jailDaysRemaining + " days remaining.","In Jail");
            return;
        }

        if(allowAudio) {
            webAppCore.playSoundEffect("chaching");
        }

        let totCost = business.managerUpCost * business.stack;

        modifyCashAmount(-totCost,false);

        business.manager++;
        business.managerUpCost = business.managerUpCost + Math.ceil(business.managerUpCost * 0.15);

        saveTime();
    }

    function numberToCash(num) {
        
        if(!num || isNaN(num)) {
            num = 0;
        }
        
        let finalNum = "$" + Math.round(num);
        
        if(num < 500) {
            finalNum = "$" + num.toFixed(2);
        }
        
        if(num >= 1000) {
            finalNum = "$" + numberWithCommas(Math.round(num));
        }
        
        return finalNum;
    }

    function numberWithCommas(x) {
        return x.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
    }

    function inGameUpdate() {
        processTick();
        updateDisplays();
        
        saveCounter++;
        
        if(saveCounter >= 500) {
            saveTime();
            saveCounter = 0;
        }
    }

    function addBusinessToFinancialListing(business) {

        let rps = getBusinessRevenuePerDay(business);

        let ourElement = document.getElementById("fin-rpt-row-" + business.businessId);

        if(ourElement) {

            let revHolder = ourElement.querySelector(".revAmount");

            if(revHolder) {

                let revColor = null;
                let revContent = numberToCash(rps);

                if(rps > 0) {
                    revColor = ManagementGame.meta.reportForegroundColor;
                } else {
                    revColor = "#B71C1C";
                }

                if(revHolder.style.color != revColor) {
                    revHolder.style.color = revColor;
                }

                if(revHolder.innerHTML != revContent) {
                    revHolder.innerHTML = revContent;
                }
            }
            


            return rps;
        }


        let tRow = document.createElement("tr");
        tRow.id = "fin-rpt-row-" + business.businessId;
                
        let tImageHolder = document.createElement("td");
        tImageHolder.className = "fReportImage";
        
        let tImage = document.createElement("img");
        tImage.src = "images/game/" + business.image;
        tImageHolder.appendChild(tImage);
        
        tRow.appendChild(tImageHolder);
        
        let tName = document.createElement("td");

        let nameText = business.customName;

        if(business.stack > 1) {
            nameText += " (x" + business.stack + ")";
        }

        tName.innerHTML = nameText;
        
        
        
        let tRev = document.createElement("td");
        tRev.className = "revAmount";
        tRev.innerHTML = numberToCash(rps);
        
        if(rps > 0) {
            tRev.style.color = ManagementGame.meta.reportForegroundColor;
        } else {
            tRev.style.color = "#B71C1C";
        }
        
        tRow.appendChild(tName);
        tRow.appendChild(tRev);
        
        tRow.businessReference = business;

        tRow.addEventListener("click",setSelectedBusinessIndexFromRow,false);
        
        tRow.className = "financialReportItem";
        
        document.getElementById("revenueReportBody").appendChild(tRow);

        return rps;
    }

    function processTick() {

        if(gamePaused || !currentGame || currentGame.businesses.length == 0) {
            return;
        }

        let totalRPS = 0;
        let totalWorkers = 0;
        
        let totalWorkersWorkers = 0;
        let totalWorkersManagers = 0;

        tmpNewStorageCap = 0;

        if(ManagementGame.meta.baseStorage) {
            tmpNewStorageCap = ManagementGame.meta.baseStorage;
        }
        
        for(let i = 0; i < currentGame.businesses.length; i++) {

            let bus = currentGame.businesses[i];

            updateBusiness(bus);

            if(!bus) {
                continue;
            }
            
            if(financialReportOpen) {

                let rps = addBusinessToFinancialListing(bus);

                totalRPS += rps;
            }
            
            totalWorkers += getEmployeeCount(bus);
            totalWorkersWorkers += bus.workers * bus.stack;
            totalWorkersManagers += bus.manager * bus.stack;
        }

        currentGame.storageCap = tmpNewStorageCap;

        if(currentGame.currentStorage > currentGame.storageCap) {
            for(let resItem in currentGame.resources) {
                if(currentGame.resources[resItem].inv > 0) {
                    currentGame.resources[resItem].inv--;
                }
            }
        }
        
        currentGame.currentDailyRevenue = totalRPS;
        currentGame.employeeCount = totalWorkers + currentGame.AccountantCount;
        currentGame.workerCount = totalWorkersWorkers;
        currentGame.managerCount = totalWorkersManagers;
        
        processOverallGameTick();

        if(financialReportOpen) {

            let reportTotal =  numberToCash(totalRPS);
            let reportMonthly =  numberToCash(totalRPS * 30);

            let revenueReportTotal = document.getElementById("revenueReportTotal");

            if(revenueReportTotal.innerHTML != reportTotal) {
                revenueReportTotal.innerHTML = reportTotal;
            }

            let revenueReportMonthly = document.getElementById("revenueReportMonthly");

            if(revenueReportMonthly.innerHTML != reportMonthly) {
                revenueReportMonthly.innerHTML = reportMonthly;
            }

        }
                
        
    }

    function setSelectedBusinessIndexFromRow(e) {
        e.stopPropagation();

        let busId = this.businessReference.businessId;

        for(let i = 0; i < currentGame.businesses.length; i++) {
            if(currentGame.businesses[i].businessId == busId) {
                currentGame.selectedBusinessIndex = i;
                break;
            }
        }

        document.getElementById("revenueReport").style.display = "none";
        financialReportOpen = false;
        webAppCore.handleBackFunction(null);

        updateDisplays();
    }

    function playCatchup() {
        
        if(!currentGame) {
            return;
        }

        let beforeCash = currentGame.cash;

        webAppCore.showSplash("Simulating Progress...");

        let timeDifferential = curTime - currentGame.savedTime;
        
        let ticksNeeded = Math.floor(timeDifferential / (currentGame.tickLength * 2));
        
        if(ticksNeeded > 15000) {
            ticksNeeded = 15000;
        }
        
        playingCatchup = true;
        
        for(let i=0;i<ticksNeeded;i++) {
            
            if(currentGame.cash < 0) {
                break;
            }
            
            processTick();
        }

        let afterCash = currentGame.cash;

        let catchupDiff = afterCash - beforeCash;

        // let player know if they gained more than 10%
        if(catchupDiff > 0 && catchupDiff > currentGame.cash * .1) {
            webAppCore.showAlertWithTitle("Great news!  You made " + numberToCash(catchupDiff) + " while you were away.  Keep those profits coming!","Ka-Ching!");
        }
        
        playingCatchup = false;
        
        gamePaused = false;
        updateDisplays();
        
        handleTimer(true);
        
        if(!webAppCore.isApeAppsLoggedIn()) {
            saveTime();
        }
    }

    function saveTime() {
        const d = new Date();
        curTime = d.getTime();
        
        currentGame.savedTime = curTime;

        for(let i = 0; i < currentGame.businesses.length; i++) {
            currentGame.businesses[i].cardHolder = null;
        }

        let gameData = JSON.stringify(currentGame);

        try {
            let encoded = btoa(gameData);
            webAppCore.saveFile("mbe_save.mbe",encoded);

        } catch(e) {
            
            webAppCore.logEvent("error","saveTime1",e);
            webAppCore.storeVar("mbe_save",gameData);

        }
    }

    function toggleMenu() {
        webAppCore.toggleMenu();
    }

    function newGame() {
        webAppCore.showChoice("Do you want to erase all saved data and start a brand new company?  This cannot be undone!","Yes","No",newGameYes,placeholder);
    }

    function newGameYes() {

        handleTimer(false);
        
        defaultOutVars();

        updateDisplays();
        
        handleTimer(true);
    }

    function defaultOutVars() {
        document.getElementById("revenueReportBody").innerHTML = "";
        currentGame = new GameData();
        gamePaused = false;

        webAppCore.showToast("Welcome to " + ManagementGame.meta.name + "!");
        
        if(!webAppCore.isApeAppsLoggedIn()) {
            saveTime();
        }
        
        updateDisplays();
        populateBusinessListing();
    }

    function showPremiumDialog() {
        var dialogOptions = {
            "title": "Premium Content",
            "message": ManagementGame.meta.name + " features a lot of free content, but to get the full experience (and remove advertising), you should consider an upgrade to the full version of the game.  Your purchase will also help fund the continued development of " + ManagementGame.meta.name + "!",
            "op1": {
                "text": "Upgrade",
                "func": function() {
                    webAppCore.presentPremium();
                },
                "color": "#4CAF50"
            },
            "op2": {
                "text": "Forget it!",
                "func": placeholder
            }
        };

        webAppCore.showDialog(dialogOptions);
    }

    function openBusiness(reference) {

        if(!reference || reference == null) {
            return;
        }

        if(reference == "techtree") {
            showTechWindow();
            return;
        }

        if(reference == "reqpremium") {
            showPremiumDialog();
            return;
        }

        if(currentGame.jailDaysRemaining > 0) {
            webAppCore.showAlertWithTitle("You cannot open a business while you are in jail!  You still have " + currentGame.jailDaysRemaining + " days remaining.","In Jail");
            return;
        }

        if(reference.cost > currentGame.cash) {
            webAppCore.showAlertWithTitle("You do not have enough money to open a new " + reference.name,"Insufficient Funds");
        } else {

            modifyCashAmount(-reference.cost,false);

            if(allowAudio) {
                webAppCore.playSoundEffect("chaching");
            }

            let biznass = new Business(reference.name,reference.image,reference.profit,reference.rate,reference.cost,reference.workers);

            biznass.businessId = currentGame.nextBusinessId;
            currentGame.nextBusinessId++;

            currentGame.businesses.push(biznass);

            currentGame.selectedBusinessIndex = currentGame.businesses.length - 1;
            
            allListed = false;

            updateDisplays();

            saveTime();

            let tipText = "";
            let runInstruction = "on the orange circle-looking run button on the toolbar";
            let arrowInstruction = "green arrow buttons on the bottom toolbar";
            let tipIcon = null;
            let buttons = [
                {
                    text: "Dismiss",
                    func: placeholder
                }
            ];

            if(webAppCore.isTV()) {
                runInstruction = "A";
                arrowInstruction = "left and right arrow buttons";
            }
            
            if(currentGame.businesses.length == 1) {
                tipText = "Congratulations on opening your first business!  You may now press " + runInstruction + " to generate revenue from this business!";
                //webAppCore.showAlertWithTitle(,"Tip");

                tipIcon = "images/ui/restart.png";

                if(webAppCore.isTV()) {
                    tipIcon = "images/ui/hud-gamepad-a.png";
                }

                if(allowAudio) {
                    playNextSong();
                }
            }


            
            if(currentGame.businesses.length == 2) {
                tipText = "Now that you own more than one business, you can use the " + arrowInstruction + " to navigate through the various companies you own.";
                //webAppCore.showAlertWithTitle("Now that you own more than one business, you can use the " + arrowInstruction + " to navigate through the various companies you own.","Tip");

                tipIcon = "images/ui/previous.png";

                if(webAppCore.isTV()) {
                    tipIcon = "images/ui/previous-no.png";
                }
            }

            if(currentGame.businesses.length == 3 && ManagementGame.technologies.length > 0) {
                tipText = "As you gain money, you can unlock new technologies that allow you to build even more businesses.  Check the technology screen in the Main Menu to see what you can unlock!";
                //webAppCore.showAlertWithTitle("As you gain money, you can unlock new technologies that allow you to build even more businesses.  Check the technology screen in the Main Menu to see what you can unlock!","Tip");

                tipIcon = "images/ui/ind-research.png";

                buttons.unshift({
                    text: "Show Technologies",
                    func: showTechWindow,
                    color: "#4CAF50"
                });
            }

            if(tipText.trim().length > 0) {
                webAppCore.showDialog({
                    title: "Tip",
                    message: tipText,
                    icon: tipIcon,
                    buttons: buttons
                });
            }
            
        }
    }
    
    function originateDebtFinanceLoan(amount) {
        originateLoan("Y" + (currentGame.year + 1).toString() + "M" + (currentGame.month + 1).toString() + " Debt Refinance Loan",amount,12);
        
        if(!playingCatchup) {
            webAppCore.showAlertWithTitle("Your accountant was forced to take out a new loan of " + numberToCash(amount) + " in order to meet the minimum payments on your previous loans!","Accounting Alert");
        }
    }

    function processQuarterEnd() {
        if(!currentGame.accounting.illegal) {
            currentGame.accounting.illegal = {
                "quarter": 0,
                "allTime": 0
            };
        }

        let quarterProfits = currentGame.accounting.revenue.quarter - currentGame.accounting.expense.quarter;
        quarterProfits -= currentGame.accounting.illegal.quarter;

        if(!currentGame.quarterlyPerformance) {
            currentGame.quarterlyPerformance = {};
        }
        
        if(!currentGame.quarterlyPerformance[currentGame.year]) {
            currentGame.quarterlyPerformance[currentGame.year] = {};
        }

        let quarter = 1;

        if(currentGame.month == 5) {
            quarter = 2;
        }

        if(currentGame.month == 8) {
            quarter = 3;
        }

        if(currentGame.month == 11) {
            quarter = 4;
        }

        
        
        currentGame.quarterlyPerformance[currentGame.year][quarter] = {
            "revenue": currentGame.accounting.revenue.quarter,
            "expense": currentGame.accounting.expense.quarter,
            "illegal": currentGame.accounting.illegal.quarter,
            "net": quarterProfits
        };

        let quarterlyTax = 0;

        if(quarterProfits > 0) {
            quarterlyTax = Math.round(quarterProfits * currentGame.corporateTaxRate);

            if(quarterlyTax > 0) {
                let accountants = currentGame.AccountantCount;

                while(accountants > 0) {

                    quarterlyTax -= Math.round(quarterlyTax * 0.05);

                    accountants--;
                }
            } else {
                quarterlyTax = 0;
            }
            
        }

        let existingTax = currentGame.taxOwed;

        currentGame.taxOwed = quarterlyTax + existingTax;

        processMonthlyPayroll(true);

        currentGame.accounting.revenue.quarter = 0;
        currentGame.accounting.expense.quarter = 0;
        currentGame.accounting.illegal.quarter = 0;
    }   

    function getTotalAnnualPayroll() {
        let totalWorkerSalary = currentGame.workerCount * currentGame.workerSalary;
        let totalManagerSalary = currentGame.managerCount * currentGame.managerSalary;
        let totalLawyerSalary = currentGame.lawyerCount * currentGame.lawyerSalary;
        let totalAccountingSalary = currentGame.AccountantCount * currentGame.accountantSalary;
        let totalInmateSalary = currentGame.jailedWorkers.length * currentGame.inmateSalary;
        let hrBudget = currentGame.hrCount * currentGame.hrSalary;
        let totalLobbySalart = currentGame.lobbyistCount * currentGame.lobbyistSalary;
        
        if(!totalWorkerSalary || isNaN(totalWorkerSalary)) {
            totalWorkerSalary = 0;
        }
        
        if(!totalManagerSalary || isNaN(totalManagerSalary)) {
            totalManagerSalary = 0;
        }
        
        if(!totalLawyerSalary || isNaN(totalLawyerSalary)) {
            totalLawyerSalary = 0;
        }
        
        if(!totalAccountingSalary || isNaN(totalAccountingSalary)) {
            totalAccountingSalary = 0;
        }
        
        if(!totalInmateSalary || isNaN(totalInmateSalary)) {
            totalInmateSalary = 0;
        }
        
        if(!hrBudget || isNaN(hrBudget)) {
            hrBudget = 0;
        }
        
        if(!totalLobbySalart || isNaN(totalLobbySalart)) {
            totalLobbySalart = 0;
        }

        return totalLobbySalart + totalWorkerSalary + totalManagerSalary + totalLawyerSalary + totalAccountingSalary + totalInmateSalary + hrBudget;
    }

    function processMonthlyPayroll(fromQuarter) {

        let totalSalary = getTotalAnnualPayroll() / 12;
        
        modifyCashAmount(-totalSalary,false);

        let inmatepay = parseInt(currentGame.inmateSalary / 12);

        if(inmatepay > 0) {
            let inmateDivTen = parseInt(inmatepay / 10);

            if(inmateDivTen < 1) {
                inmateDivTen = 1;
            }
    
            // slip money to the inmates
            for(let i = 0; i < currentGame.jailedWorkers.length; i++) {
                currentGame.jailedWorkers[i].wallet += inmatepay;
                currentGame.jailedWorkers[i].mood += randomIntFromInterval(0,inmateDivTen);
            }
        }

        let totaltax = 0;

        if(fromQuarter) {
            totaltax = currentGame.taxOwed;
            modifyCashAmount(-totaltax,false);

            currentGame.taxOwed = 0;
        }

        let neededLoan = false;

        if(currentGame.cash < 0) {
            let loanAmount = -currentGame.cash;
            originateLoan("Y" + (currentGame.year + 1).toString() + "M" + (currentGame.month + 1).toString() + " Payroll Loan",loanAmount,12);
            neededLoan = true;
        }
        
        if(!playingCatchup) {

            let msgTitle = "Payroll";

            if(fromQuarter) {
                msgTitle += " and Corporate Tax";
            }

            let msgContent = "You have just made your monthly payroll of " + numberToCash(totalSalary);

            if(fromQuarter && totaltax > 0) {
                msgContent += ".\n\nYou have also been assessed a quarterly corporate income tax from the government of " + numberToCash(totaltax);
            }

            msgContent += ", leaving you with " + numberToCash(currentGame.cash) + " in the bank.";

            if(neededLoan) {
                msgContent += "  Accounting was forced to take an emergency one year bank loan to finance your expenses.";
            }

            webAppCore.showAlertWithTitle(msgContent,msgTitle);
        }
    }
    
    function processMonthEnd() {
        if(!currentGame.historicPerformance) {
            currentGame.historicPerformance = {};
        }
        
        if(!currentGame.historicPerformance[currentGame.year]) {
            currentGame.historicPerformance[currentGame.year] = {};
        }
        
        currentGame.historicPerformance[currentGame.year][currentGame.month] = currentGame.cash;

        if(currentGame.month == 2 || currentGame.month == 5 || currentGame.month == 8 || currentGame.month == 11) {
            isQuarterEnd = true;
        }
        
    }

    function showAccountingAlertPayroll(diff) {
        if(playingCatchup) {
            return;
        }
        
        webAppCore.showAlertWithTitle("Your accountant is worried that you aren't going to be able to make this months payroll.  He is predicting you will have a budget shortfall of about " + numberToCash(diff) + ".","Accounting Alert");
    }
    
    function removeFromArray(arr, item) {
        for(let i = arr.length; i--;) {
            if(arr[i] === item) {
                arr.splice(i, 1);
            }
        }
    }
    
    function processOverallGameTick() {
        currentGame.totalTicks++;
        currentGame.dayCounter++;
        
        let i;
        
        let dayPercent = currentGame.dayCounter / ManagementGame.meta.gameTicksPerDay;
        
        currentGame.hour = Math.round(dayPercent * 23);
        
        if(currentGame.dayCounter >= ManagementGame.meta.gameTicksPerDay) {
            currentGame.dayCounter = 0;
            currentGame.day++;

            if(currentGame.jailDaysRemaining > 0) {
                currentGame.jailDaysRemaining--;

                if(!playingCatchup && currentGame.jailDaysRemaining == 0) {
                    webAppCore.showDialog({
                        title: "Freedom",
                        message: "Welcome home!  You have been released from prison.",
                        image: "images/ui/inmate.png"
                    });
                }
            } else {
                currentGame.jailDaysRemaining = 0;
            }

            processInmateDay();
            
            // loans come due on the 15th 
            if(currentGame.day == 15) {
                
                let totalDue = 0;
                let matureLoans = [];
                
                for(i = 0; i < currentGame.loans.length; i++) {
                    totalDue += currentGame.loans[i].termPayment;
                    currentGame.loans[i].maturity--;
                    currentGame.loans[i].balance -= currentGame.loans[i].termPrinciple;
                    
                    if(currentGame.loans[i].balance <= 0) {
                        matureLoans.push(currentGame.loans[i]);
                        if(!playingCatchup) {
                            webAppCore.showToast(currentGame.loans[i].name + " has been repaid in full.");
                        }
                    }
                }
                
                for(i = 0; i < matureLoans.length; i++) {
                    removeFromArray(currentGame.loans,matureLoans[i]);
                }
                
                matureLoans = [];
                
                modifyCashAmount(-totalDue,false);
                
                if(currentGame < 0) {
                    originateDebtFinanceLoan(-currentGame.cash);
                } else {
                    if(!playingCatchup) {
                        if(totalDue > 0) {
                            webAppCore.showToast("You have just made " + numberToCash(totalDue) + " in monthly loan payments.");
                        }
                    }
                }
            }
            
            // have accountants check if there are issues
            if(currentGame.day == 22) {
                let potentialEarningsUpcoming = currentGame.currentDailyRevenue * 8;
                let potentialEndOfMonth = currentGame.cash + potentialEarningsUpcoming;

                let totalSalary = getTotalAnnualPayroll() / 12;
                
                if(potentialEndOfMonth < totalSalary) {
                    let diff = totalSalary - potentialEndOfMonth;
                    showAccountingAlertPayroll(diff);
                }
            }
            
            if(currentGame.day >= 30) {
                currentGame.day = 0;

                isQuarterEnd = false;
                
                processMonthEnd();
                
                currentGame.month++;
                
                if(isQuarterEnd) {
                    processQuarterEnd();
                } else {
                    processMonthlyPayroll();
                }
                
                
                if(currentGame.month >= 12) {
                    currentGame.month = 0;
                    currentGame.year++;
                }
            }
        }
    }

    let isQuarterEnd = false;
    
    function getBusinessDefByName(name) {

        if(name == "Iron Miniong") {
            name = "Iron Mining";
        }

        for(let i = 0; i < ManagementGame.companies.length; i++) {
            if(ManagementGame.companies[i].name == name) {
                return ManagementGame.companies[i];
            }
        }
        
        return null;
    }

    function GameData() {
        this.cash = ManagementGame.meta.startingCash;
        this.name = ManagementGame.meta.name;
        this.selectedBusinessIndex = 0;
        this.savedTime = curTime;
        this.tickLength = ManagementGame.meta.defGameSpeed;
        this.totalTicks = 0;

        this.minimumWage = ManagementGame.meta.defMinimumWage;
        
        this.currentDailyRevenue = 0;
        
        this.workerSalary = ManagementGame.meta.defWorkerSalary;
        this.managerSalary = ManagementGame.meta.defManagerSalary;
        
        this.lawyerSalary = ManagementGame.meta.defLawyerSalary;
        this.accountantSalary = ManagementGame.meta.defAccountantSalary;
        this.lobbyistSalary = ManagementGame.meta.defLobbyistSalary;
        this.hrSalary = ManagementGame.meta.defHRSalary;
        this.inmateSalary = ManagementGame.meta.defInmateSalary;

        this.workerSalaryIndustry = ManagementGame.meta.defWorkerSalary;
        this.managerSalaryIndustry = ManagementGame.meta.defManagerSalary;
        this.lawyerSalaryIndustry = ManagementGame.meta.defLawyerSalary;
        this.accountantSalaryIndustry = ManagementGame.meta.defAccountantSalary;
        this.lobbyistSalaryIndustry = ManagementGame.meta.defLobbyistSalary;
        this.hrSalaryIndustry = ManagementGame.meta.defHRSalary;
        
        this.employeeCount = 0;
        this.workerCount = 0;
        this.managerCount = 0;
        
        this.lawyerCount = 0;
        this.AccountantCount = 0;
        this.lobbyistCount = 0;
        this.hrCount = 0;

        this.jailDaysRemaining = 0;
        
        this.historicPerformance = {};
        this.quarterlyPerformance = {};

        this.rapFinesPaid = 0;
        this.rapBusinessesRaided = 0;
        this.daysServed = 0;
        this.criminalCharges = {};
        this.chargesDropped = 0;
        this.jailedWorkers = [];
        
        this.corporateTaxRate = ManagementGame.meta.defCorporateTaxRate;
        
        this.year = 0;
        this.month = 0;
        this.day = 0;
        this.dayCounter = 0;
        
        this.hour = 0;

        this.businesses = [];
        this.loans = [];
        this.unlockedTechs = [];

        this.taxOwed = 0;

        this.firstResourceDepositDone = false;

        this.accounting = {
            "revenue": {
                "quarter": 0,
                "allTime": 0
            },
            "expense": {
                "quarter": 0,
                "allTime": 0
            },
            "illegal": {
                "quarter": 0,
                "allTime": 0
            }
        };

        this.nextBusinessId = 1;

        this.currentStorage = 0;
        this.storageCap = ManagementGame.meta.baseStorage;
        this.resources = {};
    }

    function JailedWorker() {
        this.days = 0;
        this.fine = 0;
        this.wallet = 0;
        this.mood = 100;
        this.crime = null;
    }

    function Resource(def) {
        this.name = def.name;
        this.price = def.basePrice;
        this.inv = def.startingInv;

        if(this.inv > 0) {
            this.isUnlocked = true;
        } else {
            this.isUnlocked = false;
        }
        
    }

    function Business(name,image,profit,rate,cost,employees) {
        this.name = name;
        this.customName = this.name;

        this.isGettingBusted = false;

        this.image = image;
        this.level = 1;
        this.manager = 0;
        this.profit = profit;
        
        this.rate = rate;
        
        if(!cost || cost == null) {
            var def = getBusinessDefByName(this.name);
            if(def != null) {
                cost = def.cost;
            }
        }
        
        this.sellValue = Math.floor(cost * 0.8);
        
        this.initialCost = cost;
        
        this.workers = employees;
        this.baseWorkers = employees;

        this.levelUpBenefit = 0.2;
        
        this.levelUpCost = Math.ceil(cost * 0.2);
        this.managerUpCost = Math.ceil(cost * 0.75);
        
        this.profitCounter = 0;
        this.isRunning = false;

        this.uiMeter = null;
        this.card = null;
        this.cardHolder = null;

        this.businessId = 0;

        // to be added, just clone/stack another business here and double all stats
        this.stack = 1;
    }
    
    function getModifiedRate(business) {
        if(!business.baseWorkers) {
            business.baseWorkers = business.workers;
        }
        
        let modRate = business.rate;
        
        let extraWorkers = getEmployeeCount(business) - business.baseWorkers;
        modRate -= extraWorkers;
        
        if(modRate < 1) {
            modRate = 1;
        }
        
        return modRate;
    }
    
    function getModifiedProfit(business) {
        
        if(!business.baseWorkers) {
            business.baseWorkers = business.workers;
        }
        
        let modProfit = business.profit;
        
        let extraProfitPerWorker = business.profit * ManagementGame.meta.profitModWorker;
        let extraProfitPerManager = business.profit * ManagementGame.meta.profitModManager;
        
        let i;
        
        // add extra profit for managers
        for(i = 0; i < business.manager; i++) {
            modProfit += extraProfitPerManager;
            
            extraProfitPerManager = extraProfitPerManager * ManagementGame.meta.profitModManagerAdditionalMod;
        }
        
        // add extra profit for workers over the base
        let extraWorkers = business.workers - business.baseWorkers;
        if(extraWorkers > 0) {
            for(i = 0; i < extraWorkers; i++) {
                modProfit += extraProfitPerWorker;
                extraProfitPerWorker = extraProfitPerWorker * ManagementGame.meta.profitWorkerAdditionalMod;
            }
        }
        
        // subtract profits for worker deficiency
        if(extraWorkers < 0) {
            extraProfitPerWorker = business.profit * 0.15;
            extraWorkers *= -1;
            for(i = 0; i < extraWorkers; i++) {
                modProfit -= extraProfitPerWorker;
                extraProfitPerWorker = extraProfitPerWorker * 1.15;
            }
        }

        modProfit *= business.stack;
        
        return modProfit;
    }

    function redrawFinancialReport() {
        document.getElementById("revenueReportBody").innerHTML = "";
    }
 
    function getEmployeeCount(business) {

        if(!business) {
            return 0;
        }

        return (business.workers + business.manager) * business.stack;
    }

    function getResourceDefByName(name) {
        for(let i = 0; i < ManagementGame.resources.length; i++) {
            if(ManagementGame.resources[i].name == name) {
                return ManagementGame.resources[i];
            }
        }

        return null;
    }

    function getCrimeByName(crime) {
        for(let i = 0; i < ManagementGame.crimes.length; i++) {
            if(ManagementGame.crimes[i].name == crime) {
                return ManagementGame.crimes[i];
            }
        }

        return null;
    }

    function shutDownBusiness(business,crime) {
        

        let i;

        if(playingCatchup) {
            return;
        }

        // for each lawyer, a 10% chance of charges being dropped
        for(i = 0; i < currentGame.lawyerCount; i++) {
            let dropChance = randomIntFromInterval(0,100);
            if(dropChance < 10) {
                currentGame.chargesDropped++;
                return;
            }
        }

        

        let newJailedWorkers = [];

        let bdef = getBusinessDefByName(business.name);

        if(!bdef) {
            return;
        }

        let crimeDef = getCrimeByName(crime);

        if(!crimeDef) {
            return;
        }

        business.isGettingBusted = true;

        let totalToJail = bdef.workers * business.stack;
        totalToJail += (business.manager * business.stack);

        let lawyerReductionChance = 0.05 * currentGame.lawyerCount;
        if(lawyerReductionChance > 0.75) {
            lawyerReductionChance = 0.75;
        }

        let totalFine = 0;

        for(i = 0; i < totalToJail; i++) {

            let fine = 0;
            let time = 0;

            if(crimeDef.fine > 0) {
                fine = randomIntFromInterval(0,crimeDef.fine - Math.round(crimeDef.fine * lawyerReductionChance));
            }

            if(crimeDef.days > 0) {
                time = randomIntFromInterval(0,crimeDef.days - Math.round(crimeDef.days * lawyerReductionChance));
            }

            totalFine += fine;

            if(fine > 0 || time > 0) {
                let criminal = new JailedWorker();

                if(time < 1) {
                    time = 1;
                }

                criminal.days = time;
                criminal.fine = fine;
                criminal.crime = crime;

                // eventually base this on pay rate
                criminal.mood = randomIntFromInterval(90,110);

                newJailedWorkers.push(criminal);
            }

            
        }

        currentGame.rapBusinessesRaided++;
        removeFromArray(currentGame.businesses,business);
        redrawFinancialReport();

        let msg = "Authorities have just raided our " + business.name + " business!";

        if(newJailedWorkers.length > 0) {
            msg += "  " + numberWithCommas(newJailedWorkers.length) + " employees have been convicted of " + crime + " and will be doing time.";
        }

        msg += "  The business has been shut down!";

        if(totalFine > 0 && totalFine < currentGame.cash) {
            msg += "  The workers are facing legal fines totaling " + numberToCash(totalFine) + ".  Will you be covering the cost?";

            webAppCore.showDialog({
                title: business.customName + " Busted!",
                message: msg,
                image: "images/ui/inmate.png",
                op1: {
                    text: "Yes",
                    func: function() {
                        modifyCashAmount(-totalFine,true);
                        currentGame.rapFinesPaid += totalFine;

                        for(i = 0; i < newJailedWorkers.length; i++) {
                            newJailedWorkers[i].fine = 0;
                            newJailedWorkers[i].mood += 15;
                        }

                        sendWorkersToJail(newJailedWorkers);
                    }
                },
                op2: {
                    text: "No",
                    func: function() {
                        for(i = 0; i < newJailedWorkers.length; i++) {
                            newJailedWorkers[i].mood -= 35;

                            if(newJailedWorkers[i].mood < 0) {
                                newJailedWorkers[i].mood = 0;
                            }
                        }

                        sendWorkersToJail(newJailedWorkers);
                    }
                }
            });
        } else {
            webAppCore.showDialog({
                title: business.customName + " Busted!",
                message: msg,
                image: "images/ui/inmate.png"
            });

            for(i = 0; i < newJailedWorkers.length; i++) {
                newJailedWorkers[i].mood -= 15;

                if(newJailedWorkers[i].mood < 0) {
                    newJailedWorkers[i].mood = 0;
                }
            }

            sendWorkersToJail(newJailedWorkers);
        }

        
    }

    function sendWorkersToJail(workers) {

        if(!workers) {
            return;
        }

        for(let i = 0; i < workers.length; i++) {
            currentGame.jailedWorkers.push(workers[i]);
        }

    }

    function updateBusiness(business) {
        let i,genItem;

        if(business.isGettingBusted) {
            return;
        }

        let bdef = getBusinessDefByName(business.name);

        if(business.isRunning) {
            business.profitCounter++;
            
            if(business.profitCounter > getModifiedRate(business)) {

                if(bdef && bdef.oneOffJob && playingCatchup) {
                    business.profitCounter = getModifiedRate(business);
                    return;
                }
                

                if(bdef) {
                    // first, there is a chance for the law to come down on the business
                    if(bdef.crimeLevels) {
                        let lawyerReduction = parseInt(currentGame.lawyerCount) * 10;
                        let topRisk = 200;

                        if(business.manager == 0) {
                            topRisk = 140;
                        }

                        // for new games, make the chance of getting busted a lot lower
                        if(currentGame.businesses.length < 5) {
                            topRisk += 600;
                        }

                        for(i = 0; i < bdef.crimeLevels.length; i++) {
                            let bustChance = randomIntFromInterval(0,topRisk + lawyerReduction);

                            if(bustChance < bdef.crimeLevels[i].risk) {
                                shutDownBusiness(business,bdef.crimeLevels[i].crime);
                                return;
                            }
                        }
                    }
                }


                business.profitCounter = 0;
                business.isRunning = false;

                let illegalBusiness = false;

                if(bdef && bdef.crimeLevels && bdef.crimeLevels.length > 0) {
                    illegalBusiness = true;
                }

                modifyCashAmount(getModifiedProfit(business),illegalBusiness);

                

                if(bdef && bdef.resourceGen && bdef.resourceGen.output && bdef.resourceGen.output.length > 0) {
                    for(i = 0; i < bdef.resourceGen.output.length; i++) {
                        genItem = bdef.resourceGen.output[i];
                        modifyResourceAmount(genItem.resource,genItem.amount * business.stack);
                    }
                }


                if(bdef && bdef.oneOffJob) {
                    removeFromArray(currentGame.businesses,business);
                    redrawFinancialReport();
                }

                
                
            }
        } else {
            if(business.manager > 0 || bdef.runsWithoutManagement) {
                let min = 1;
                let max = 18;
                
                let change = Math.floor(Math.random() * max) + min;

                let checkLvl = business.manager;

                if(bdef.runsWithoutManagement) {
                    checkLvl = 1;
                }
                
                if(change <= checkLvl) {
                    runBusiness(business);
                }
            } else {

                let chanceMax = 250;

                if(playingCatchup) {
                    chanceMax = 2000;
                }

                if(bdef) {
                    if(bdef.profit == 0 && (!bdef.resourceGen || bdef.resourceGen && bdef.resourceGen.output.length == 0)) {
                        chanceMax = 10;
                    }
                }

                let runChance = randomIntFromInterval(0,chanceMax);
                if(runChance == 5) {
                    runBusiness(business);
                }
            }
        }

        if(bdef && bdef.storageMod) {
            tmpNewStorageCap += (parseInt(bdef.storageMod) * business.stack);
        }
    }

    function businessUpdatePercent(business) {
        let updatePercentRaw = business.profitCounter / getModifiedRate(business);
        let finalPercent = Math.round(updatePercentRaw * 100);
        
        return finalPercent;
    }

    function runBusiness(business) {
        if(business.isRunning) {
            webAppCore.showToast("Already Running!");
            return;
        }
        
        business.profitCounter = 0;

        let bdef = getBusinessDefByName(business.name);

        let i,genItem;
        let hasInputRequirement = true;

        if(bdef && bdef.resourceGen && bdef.resourceGen.input && bdef.resourceGen.input.length > 0) {
            for(i = 0; i < bdef.resourceGen.input.length; i++) {
                genItem = bdef.resourceGen.input[i];

                if(!currentGame.resources[genItem.resource]) {
                    currentGame.resources[genItem.resource] = new Resource(getResourceDefByName(genItem.resource));
                }

                

                if(genItem.amount * business.stack > currentGame.resources[genItem.resource].inv) {
                    hasInputRequirement = false;
                    break;
                }
            }
        }

        if(hasInputRequirement && bdef && bdef.resourceGen && bdef.resourceGen.input && bdef.resourceGen.input.length > 0) {
            for(i = 0; i < bdef.resourceGen.input.length; i++) {
                genItem = bdef.resourceGen.input[i];
                modifyResourceAmount(genItem.resource,-(genItem.amount * business.stack));
            }
        }

        if(!hasInputRequirement) {
            business.isRunning = false;
            return;
        }

        business.isRunning = true;
    }
    
    function getBusinessRevenuePerDay(business) {

        let modRate = getModifiedRate(business);

        let managerPenalty = 0;

        if(business.manager == 0) {
            managerPenalty = ManagementGame.meta.gameTicksPerDay;
        }

        if(business.manager == 1) {
            managerPenalty = 20;
        }

        if(business.manager == 2) {
            managerPenalty = 10;
        }

        if(business.manager == 3) {
            managerPenalty = 5;
        }

        if(business.manager == 4) {
            managerPenalty = 1;
        }

        modRate += managerPenalty;

        let probableTransactionsPerDay = ManagementGame.meta.gameTicksPerDay / modRate;
        let revenuePerDay = probableTransactionsPerDay * getModifiedProfit(business);

        let monthlyWorkerCost = (business.workers * currentGame.workerSalary) / 12;
        let monthlyManagerCost = (business.manager * currentGame.managerSalary) / 12;

        
        monthlyWorkerCost *= business.stack;
        monthlyManagerCost *= business.stack;
        
        
        let dailyWorkerCost = monthlyWorkerCost / 30;
        let dailyManagerCost = monthlyManagerCost / 30;
        
        revenuePerDay -= dailyWorkerCost;
        revenuePerDay -= dailyManagerCost;


        return revenuePerDay;
    }

    function toggleFinancial() {
        let revenueReport = document.getElementById("revenueReport");
        if(revenueReport.style.display == "block") {
            revenueReport.style.display = "none";
            financialReportOpen = false;
            webAppCore.handleBackFunction(null);
        } else {
            revenueReport.style.display = "block";
            financialReportOpen = true;
            
            webAppCore.handleBackFunction(onBack);

        }
    }
    
    function showHR() {
        let hrOptions = [];


        hrOptions.push({
            title: "Labor Costs",
            description: "The primary expense in the company is labor.  You will need to keep costs down while still maintaining morale and productivity.",
            icon: "images/ui/money.png",
            overflow: [
                {
                    pre: "Annual Labor Costs: ",
                    post: numberToCash(getTotalAnnualPayroll())
                },
                {
                    pre: "Monthly Labor Costs: ",
                    post: numberToCash(getTotalAnnualPayroll() / 12)
                }
            ],
            tag: "laborcosts"
        });

        // defHRSalary

        let hrBudget = currentGame.hrCount * currentGame.hrSalary;


        hrOptions.push({
            title: "Human Resource Managers",
            description: "You have " + numberWithCommas(currentGame.hrCount) + " full time Human Resource managers on staff.  HR managers can improve employee relations and help avoid worker strikes.  You need at least one HR manager to make wage level adjustments.",
            icon: "images/ui/hr.png",
            status: "Annual Expenses: " + numberToCash(hrBudget),
            overflow: [
                {
                    pre: "Total Workforce: ",
                    post: numberWithCommas(currentGame.hrCount)
                },
                {
                    pre: "Salary: ",
                    post: numberToCash(currentGame.hrSalary)
                }
            ],
            tag: "staff"
        });

        let totalWorkerSalary = currentGame.workerCount * currentGame.workerSalary;
        

        hrOptions.push({
            title: "Workers",
            description: "These are the regular blue collar front-line employees of the company.  Maintaining a happy workforce will improve efficiency and reduce or eliminate the occurance of strikes.",
            icon: "images/ui/worker.png",
            status: "Annual Expenses: " + numberToCash(totalWorkerSalary),
            overflow: [
                {
                    pre: "Total Workforce: ",
                    post: numberWithCommas(currentGame.workerCount)
                },
                {
                    pre: "Salary: ",
                    post: numberToCash(currentGame.workerSalary)
                }
            ],
            tag: "workers"
        });


        let totalManagerSalary = currentGame.managerCount * currentGame.managerSalary;

        hrOptions.push({
            title: "Managers",
            description: "Managers set your workforce to task and allow businesses to run automatically without player input, even while the game is inactive.  They also improve the efficiency and profits of a business.",
            icon: "images/ui/manager.png",
            status: "Annual Expenses: " + numberToCash(totalManagerSalary),
            overflow: [
                {
                    pre: "Total Workforce: ",
                    post: numberWithCommas(currentGame.managerCount)
                },
                {
                    pre: "Salary: ",
                    post: numberToCash(currentGame.managerSalary)
                }
            ],
            tag: "managers"
        });


        let totalLawyerSalary = currentGame.lawyerCount * currentGame.lawyerSalary;

        if(ManagementGame.crimes.length > 0) {
            hrOptions.push({
                title: "Lawyers",
                description: "Lawyers help keep the law at bay by preventing business raids and representing your employees (and yourself) in the case of a raid or arrest.  A well staffed legal department can keep even the most crooked business away from government scrutiny.",
                icon: "images/ui/lawyer.png",
                status: "Annual Expenses: " + numberToCash(totalLawyerSalary),
                overflow: [
                    {
                        pre: "Total Workforce: ",
                        post: numberWithCommas(currentGame.lawyerCount)
                    },
                    {
                        pre: "Salary: ",
                        post: numberToCash(currentGame.lawyerSalary)
                    }
                ],
                tag: "lawyers"
            });
        }


        let totalAccountingSalary = currentGame.AccountantCount * currentGame.accountantSalary;

        hrOptions.push({
            title: "Accountants",
            description: "The bean counters of the organization.  A skilled accounting team can reduce your legal tax burden dramatically and help keep government regulators out of your books.",
            icon: "images/ui/accountant.png",
            status: "Annual Expenses: " + numberToCash(totalAccountingSalary),
            overflow: [
                {
                    pre: "Total Workforce: ",
                    post: numberWithCommas(currentGame.AccountantCount)
                },
                {
                    pre: "Salary: ",
                    post: numberToCash(currentGame.accountantSalary)
                }
            ],
            tag: "accountants"
        });



        let totalInmateSalary = currentGame.jailedWorkers.length * currentGame.inmateSalary;

        if(ManagementGame.crimes.length > 0) {
            hrOptions.push({
                title: "Inmates",
                description: "Workers who are no longer technically employed by the company, but have been arrested and sent to prison while on the job.  You have no legal obligation to pay them, but keeping them on the payroll during their incarceration can help them settle their legal fees and buy their loyalty, preventing them from cooperating with the authorities and implicating the company.",
                icon: "images/ui/inmate.png",
                status: "Annual Expenses: " + numberToCash(totalInmateSalary),
                overflow: [
                    {
                        pre: "Serving Time: ",
                        post: numberWithCommas(currentGame.jailedWorkers.length)
                    },
                    {
                        pre: "Stipend: ",
                        post: numberToCash(currentGame.inmateSalary)
                    }
                ],
                tag: "inmates"
            });
        }

        webAppCore.presentNativeList("Human Resources",hrOptions,onHRoptionSelected,"Dismiss");
    }

    function onHRoptionSelected(tag) {

        if(tag == "accountants") {
            hireAccountantDialog();
        }

        if(tag == "staff") {
            hrStaffingDialog();
        }

        if(tag == "lawyers") {
            legalStaffingDialog();
        }

        if(tag == "workers") {
            workerSalaryDialog();
        }

        if(tag == "managers") {
            managerSalaryDialog();
        }

        if(tag == "inmates") {
            inmateSalaryDialog();
        }
    }

    function showEmployees() {
        
        if(currentGame == null) {
            return;
        }

        showHR();

    }

    function renameSelected() {
        if(gamePaused) {
            onResume();
        }
        
        if(currentGame.businesses.length == 0) {
            webAppCore.showAlertWithTitle("You must start a new business first before you can name it!","No business!");
            return;
        }
        
        let ourIndex = currentGame.selectedBusinessIndex;
        
        if(isNaN(ourIndex) || ourIndex >= currentGame.businesses.length) {
            ourIndex = 0;
        }
        
        currentGame.selectedBusinessIndex = ourIndex;
        
        let currentName = currentGame.businesses[ourIndex].name;
        
        webAppCore.inputBox("Please enter a new name for " + currentName,processNewName);
    }

    function processNewName(value) {

        if(!value || value.trim().length == 0) {
            webAppCore.showAlertWithTitle("The name you have picked is not valid.","Invalid Name");
            return;
        }

        value = value.trim().latinize();
        
        let ourIndex = currentGame.selectedBusinessIndex;
        
        if(isNaN(ourIndex) || ourIndex >= currentGame.businesses.length) {
            ourIndex = 0;
        }
        
        currentGame.selectedBusinessIndex = ourIndex;
        
        currentGame.businesses[ourIndex].customName = value;
        saveTime();

        redrawFinancialReport();
        updateDisplays();
    }
    
    function originateLoan(loanName,loanAmount,loanTerm) {
        
        let loan = buildLoan(loanName,loanAmount,loanTerm);
        
        modifyCashAmount(loanAmount,false);
        currentGame.loans.push(loan);
    }

    function buildLoan(loanName,loanAmount,loanTerm) {
        let loan = {};
        
        loan.name = loanName;
        loan.balance = loanAmount;
        loan.maturity = loanTerm;
        
        
        loan.rate = ManagementGame.meta.baseLoanRate;
        
        for(let i = 0; i < currentGame.loans.length; i++) {
            loan.rate += 0.005;
        }

        loan.rate += (0.0001 * loanTerm);
        
        loan.termInterest = (loanAmount * loan.rate) / loanTerm;
        loan.termPrinciple = loanAmount / loanTerm;
        loan.termPayment = loan.termInterest + loan.termPrinciple;

        return loan;
    }
    
    function showLoanList() {

        if(currentGame == null) {
            return;
        }
        
        let loanItems = [];

        let loanItem = {};
        loanItem.title = "New Loan";
        loanItem.description = "Take out a new loan from The Bank.";
        loanItem.status = "The Bank is an equal opportunity lender.";
        loanItem["status-color"] = "#4CAF50";
        loanItem.tag = "newloan";
        loanItems.push(loanItem);
        
        for(let i = 0; i < currentGame.loans.length; i++) {
            loanItem = {};
            loanItem.title = currentGame.loans[i].name;
            loanItem["title-color"] = null;
            loanItem.description = "Outstanding Principal: " + numberToCash(currentGame.loans[i].balance) + " (" + currentGame.loans[i].maturity + " payments remaining, " + (currentGame.loans[i].rate * 100) + "% interest)";
            loanItem.status = "Monthly Payment: " + numberToCash(currentGame.loans[i].termPayment);
            loanItem["status-color"] = null;
            loanItem.icon = null;
            loanItem.tag = currentGame.loans[i];
            loanItems.push(loanItem);
        }
        
        webAppCore.presentNativeList("Loans",loanItems,onLoanSelected,"Dismiss");
    }
    
    let pendingLoan;
    
    function closeOutPendingLoan() {
        if(!pendingLoan) {
            return;
        }

        
        
        if(currentGame.cash < pendingLoan.balance) {
            return;
        }
        
        modifyCashAmount(-pendingLoan.balance,false);
        
        webAppCore.showToast(pendingLoan.name + " has been repaid in full.");
        
        removeFromArray(currentGame.loans,pendingLoan);
    }
    
    function onLoanSelected(loan) {
        if(!loan || loan == null || loan.balance > currentGame.cash) {
            return;
        }

        if(loan == "newloan") {
            getNewLoanStatus();
            return;
        }
        
        pendingLoan = loan;
        
        webAppCore.showChoice("Do you want to close out this loan for " + numberToCash(loan.balance) + "?","Yes","No",closeOutPendingLoan,placeholder);
    }
    
    let pendingSellBusiness;
    let pendingSellValue;
    
    function sellBusiness(business) {

        if(currentGame.jailDaysRemaining > 0) {
            webAppCore.showAlertWithTitle("You cannot sell a business while you are in jail!  You still have " + currentGame.jailDaysRemaining + " days remaining.","In Jail");
            return;
        }
        
        let sellValue = getBusinessRevenuePerDay(business) * 8;
        
        if(business.manager == 0) {
            sellValue = Math.floor(sellValue / 4);
        }
        
        if(business.sellValue && business.sellValue > 0) {
            sellValue = business.sellValue;
        }
        
        sellValue *= business.stack;
        
        pendingSellBusiness = business;
        pendingSellValue = sellValue;
        
        webAppCore.showChoice("You can sell this business for " + numberToCash(pendingSellValue) + ".  Do you want to do it?","Yes","No",finalizeSell,placeholder);
    }
    
    function placeholder() {
        
    }
    
    function finalizeSell() {
        if(!pendingSellBusiness) {
            return;
        }

        let ourElement = document.getElementById("fin-rpt-row-" + pendingSellBusiness.businessId);

        if(ourElement) {
            document.getElementById("revenueReportBody").removeChild(ourElement);
        }
        
        removeFromArray(currentGame.businesses,pendingSellBusiness);
        modifyCashAmount(pendingSellValue,false);
        
        if(currentGame.selectedBusinessIndex > currentGame.businesses.length - 1) {
            currentGame.selectedBusinessIndex--;
        }
        
        if(currentGame.selectedBusinessIndex < 0) {
            currentGame.selectedBusinessIndex = 0;
        }

        updateDisplays();
    }

    function onApeAppsStatusChange(status) {
        populateBusinessListing();
        
        if(status) {
            webAppCore.showSplash("Syncing...");
            loadGame();
        }
    }

    function showAccountingScreen() {
        let accountingItems = [];

        let accountingBudget = currentGame.AccountantCount * currentGame.accountantSalary;

        accountingItems.push({
            "title": "Accounting Department",
            "description": "You have " + numberWithCommas(currentGame.AccountantCount) + " full time accountants on staff.  Accountants reduce the amount of tax you owe the government each quarter.",
            "icon": "images/ui/accountant.png",
            "status": "Annual Expenses: " + numberToCash(accountingBudget),
            "tag": "staff"
        });


        let estimatedTax = Math.round((currentGame.accounting.revenue.quarter - currentGame.accounting.expense.quarter) * currentGame.corporateTaxRate);

        let afterAccountant = 0;

        if(estimatedTax < 0) {
            estimatedTax = 0;
        } else {
            let accountants = currentGame.AccountantCount;
            afterAccountant = estimatedTax;
            
            while(accountants > 0) {

                afterAccountant -= Math.round(afterAccountant * 0.05);

                accountants--;
            }
        }

        let profitColor = null;
        let accountantColor = null;
        let realProfitColor = null;

        if(afterAccountant < estimatedTax) {
            accountantColor = "#4CAF50";
        }

        if((currentGame.accounting.revenue.quarter + currentGame.accounting.illegal.quarter) - currentGame.accounting.expense.quarter > 0) {
            realProfitColor = "#4CAF50";
        }

        if((currentGame.accounting.revenue.quarter + currentGame.accounting.illegal.quarter) - currentGame.accounting.expense.quarter < 0) {
            realProfitColor = "#F44336";
        }

        if(currentGame.accounting.revenue.quarter - currentGame.accounting.expense.quarter > 0) {
            profitColor = "#4CAF50";
        }

        if(currentGame.accounting.revenue.quarter - currentGame.accounting.expense.quarter < 0) {
            profitColor = "#F44336";
        }

        accountingItems.push({
            "title": "Quarterly Report",
            "description": "Every quarter we must pay the government a " + Math.round(currentGame.corporateTaxRate * 100) + "% tax on our profits.  Here is how this quarter is shaping up.",
            "icon": "images/ui/star.png",
            "overflow": [
                {
                    "pre": "Legal Revenue: ",
                    "post": numberToCash(currentGame.accounting.revenue.quarter)
                },
                {
                    "pre": "Illegal Revenue: ",
                    "post": numberToCash(currentGame.accounting.illegal.quarter)
                },
                {
                    "pre": "Expense: ",
                    "post": numberToCash(currentGame.accounting.expense.quarter)
                },
                {
                    "pre": "Profit: ",
                    "post": numberToCash((currentGame.accounting.revenue.quarter + currentGame.accounting.illegal.quarter) - currentGame.accounting.expense.quarter),
                    "color": realProfitColor
                },
                {
                    "pre": "Reported Profit: ",
                    "post": numberToCash(currentGame.accounting.revenue.quarter - currentGame.accounting.expense.quarter),
                    "color": profitColor
                },
                {
                    "pre": "Estimated Tax: ",
                    "post": numberToCash(estimatedTax)
                },
                {
                    "pre": "Estimated Tax After Accounting: ",
                    "post": numberToCash(afterAccountant),
                    "color": accountantColor
                }
            ],
            "tag": "quarterly"
        });

        let totalLoans = 0;

        for(let i = 0; i < currentGame.loans.length; i++) {
            totalLoans += currentGame.loans[i].balance;
        }

        let loanColor = null;

        if(totalLoans > 0) {
            loanColor = "#F44336";
        }

        accountingItems.push({
            "title": "Loans",
            "description": "You can borrow money or repay your outstanding loans",
            "icon": "images/ui/money.png",
            "status": "Outstanding Debt: " + numberToCash(totalLoans),
            "status-color": loanColor,
            "tag": "loans"
        });

        webAppCore.presentNativeList("Accounting",accountingItems,onAccountingItemSelected,"Dismiss");
    }

    function onAccountingItemSelected(tag) {

        if(tag == "staff") {
            hireAccountantDialog();
        }

        if(tag == "loans") {
            setTimeout(showLoanList,200);
        }
    }

    function legalStaffingDialog() {
        if(!webAppCore.getIsPremium()) {
            showPremiumDialog();
            return;
        }

        let legalOptions = [];



        legalOptions.push({
            "title": "Hire Lawyer",
            "description": "Hiring a company-wide Lawyer is free, but you must pay their annual salary of " + numberToCash(currentGame.lawyerSalary) + ".  Lawyers defend you, the company, and your employees from all legal risks.",
            "icon": "images/ui/add.png",
            "tag": "hire"
        });

        if(currentGame.lawyerCount > 0) {
            legalOptions.push({
                "title": "Fire Lawyer",
                "description": "Save money by downsizing the legal team.",
                "icon": "images/ui/subtract.png",
                "tag": "fire"
            });
        }

        if(currentGame.hrCount > 0) {
            legalOptions.push({
                "title": "Adjust Salary",
                "description": "Make changes to the current Lawyer salary rate.",
                "icon": "images/ui/money.png",
                "overflow": [
                    {
                        pre:    "Current: ",
                        post:   numberToCash(currentGame.lawyerSalary)
                    },
                    {
                        pre:    "National Average: ",
                        post:   numberToCash(currentGame.lawyerSalaryIndustry)
                    }
                ],
                "tag": "salary"
            });
        }

        webAppCore.presentNativeList("Legal Team",legalOptions,onLegalStaffItemSelected,"Dismiss");
    }

    function onLegalStaffItemSelected(tag) {
        if(!tag) {
            return;
        }

        if(tag == "hire") {
            currentGame.lawyerCount += 1;
            let hrBudget = currentGame.lawyerCount * currentGame.lawyerSalary;
            webAppCore.showAlertWithTitle("You have added a Lawyer to the legal team.  Annual costs at the Legal department are now " + numberToCash(hrBudget) + ".");
        }

        if(tag == "fire") {
            currentGame.lawyerCount -= 1;

            if(currentGame.lawyerCount < 0) {
                currentGame.lawyerCount = 0;
            }

            let hrBudget2 = currentGame.lawyerCount * currentGame.lawyerSalary;
            webAppCore.showAlertWithTitle("You have downsized the legal team.  Annual costs at the Legal department are now " + numberToCash(hrBudget2) + ".");
        }

        if(tag == "salary") {
            webAppCore.inputBox({
                title: "Salary",
                message: "Enter the new annual salary for the HR department.  Current government mandated minimum salary is  " + numberToCash(currentGame.minimumWage) + ".",
                func: function(amt) {
                    if(!amt || amt.trim().length == 0 || isNaN(amt)) {
                        webAppCore.showToast("Invalid Wage!");
                        return;
                    }

                    let wageInt = parseInt(amt);

                    if(wageInt < currentGame.minimumWage) {
                        webAppCore.showToast("Cannot pay below minimum wage!");
                        return;
                    }

                    currentGame.lawyerSalary = wageInt;

                    webAppCore.showToast({
                        message: "Lawyer Wage Set: " + numberToCash(amt),
                        image: "images/ui/lawyer.png",
                        action: {
                            title: "Show",
                            func: showHR
                        }
                    });
                }
            });
        }
    }

    function hrStaffingDialog() {
        if(!webAppCore.getIsPremium()) {
            showPremiumDialog();
            return;
        }

        let hrOptions = [];



        hrOptions.push({
            "title": "Hire HR Manager",
            "description": "Hiring a company-wide HR Manager is free, but you must pay their annual salary of " + numberToCash(currentGame.hrSalary) + ".  HR managers allow you to make adjustment to company wide pay scales and help improve employee relations.",
            "icon": "images/ui/add.png",
            "tag": "hire"
        });

        if(currentGame.AccountantCount > 0) {
            hrOptions.push({
                "title": "Fire HR Manager",
                "description": "Save money by downsizing the HR staff.",
                "icon": "images/ui/subtract.png",
                "tag": "fire"
            });
        }

        if(currentGame.hrCount > 0) {
            hrOptions.push({
                "title": "Adjust Salary",
                "description": "Make changes to the current HR Manager salary rate.",
                "icon": "images/ui/money.png",
                "overflow": [
                    {
                        pre:    "Current: ",
                        post:   numberToCash(currentGame.hrSalary)
                    },
                    {
                        pre:    "National Average: ",
                        post:   numberToCash(currentGame.hrSalaryIndustry)
                    }
                ],
                "tag": "salary"
            });
        }

        webAppCore.presentNativeList("Human Resources Staff",hrOptions,onHRStaffItemSelected,"Dismiss");
    }

    function onHRStaffItemSelected(tag) {
        if(!tag) {
            return;
        }

        if(tag == "hire") {
            currentGame.hrCount += 1;
            let hrBudget = currentGame.hrCount * currentGame.hrSalary;
            webAppCore.showAlertWithTitle("You have added an HR manager to the staff.  Annual costs at the Human Resources department are now " + numberToCash(hrBudget) + ".");
        }

        if(tag == "fire") {
            currentGame.hrCount -= 1;

            if(currentGame.hrCount < 0) {
                currentGame.hrCount = 0;
            }

            let hrBudget2 = currentGame.hrCount * currentGame.hrSalary;
            webAppCore.showAlertWithTitle("You have downsized the HR staff.  Annual costs at the Human Resources department are now " + numberToCash(hrBudget2) + ".");
        }

        if(tag == "salary") {
            webAppCore.inputBox({
                title: "Salary",
                message: "Enter the new annual salary for the HR department.  Current government mandated minimum salary is  " + numberToCash(currentGame.minimumWage) + ".",
                func: function(amt) {
                    if(!amt || amt.trim().length == 0 || isNaN(amt)) {
                        webAppCore.showToast("Invalid Wage!");
                        return;
                    }

                    let wageInt = parseInt(amt);

                    if(wageInt < currentGame.minimumWage) {
                        webAppCore.showToast("Cannot pay below minimum wage!");
                        return;
                    }

                    currentGame.hrSalary = wageInt;

                    webAppCore.showToast({
                        message: "HR Wage Set: " + numberToCash(amt),
                        image: "images/ui/hr.png",
                        action: {
                            title: "Show",
                            func: showHR
                        }
                    });
                }
            });
        }
    }

    function hireAccountantDialog() {

        if(!webAppCore.getIsPremium()) {
            showPremiumDialog();
            return;
        }

        let accountingOptions = [];



        accountingOptions.push({
            "title": "Hire Accountant",
            "description": "Hiring a company-wide accountant is free, but you must pay their annual salary of " + numberToCash(currentGame.accountantSalary) + ".  Each accountant you hire will reduce the amount of corporate tax you must pay.  You must balance your tax savings with the cost of employing additional accountants.",
            "icon": "images/ui/add.png",
            "tag": "hire"
        });

        if(currentGame.AccountantCount > 0) {
            accountingOptions.push({
                "title": "Fire Accountant",
                "description": "Save money by downsizing the accounting staff.",
                "icon": "images/ui/subtract.png",
                "tag": "fire"
            });
        }

        if(currentGame.hrCount > 0) {
            accountingOptions.push({
                "title": "Adjust Salary",
                "description": "Make changes to the current Accounting salary rate.",
                "icon": "images/ui/money.png",
                "overflow": [
                    {
                        pre:    "Current: ",
                        post:   numberToCash(currentGame.accountantSalary)
                    },
                    {
                        pre:    "National Average: ",
                        post:   numberToCash(currentGame.accountantSalaryIndustry)
                    }
                ],
                "tag": "salary"
            });
        }

        webAppCore.presentNativeList("Accounting Staff",accountingOptions,onAccountingStaffItemSelected,"Dismiss");
    }

    function onAccountingStaffItemSelected(tag) {
        if(!tag) {
            return;
        }

        if(tag == "hire") {
            addAccountant();
        }

        if(tag == "fire") {
            currentGame.AccountantCount -= 1;

            if(currentGame.AccountantCount < 0) {
                currentGame.AccountantCount = 0;
            }

            let accountingBudget = currentGame.AccountantCount * currentGame.accountantSalary;
            webAppCore.showAlertWithTitle("You have downsized the Accounting staff.  Annual costs at the Accounting department are now " + numberToCash(accountingBudget) + ".");
        }

        if(tag == "salary") {
            webAppCore.inputBox({
                title: "Salary",
                message: "Enter the new annual salary for the Accounting department.  Current government mandated minimum salary is  " + numberToCash(currentGame.minimumWage) + ".",
                func: function(amt) {
                    if(!amt || amt.trim().length == 0 || isNaN(amt)) {
                        webAppCore.showToast("Invalid Wage!");
                        return;
                    }

                    let wageInt = parseInt(amt);

                    if(wageInt < currentGame.minimumWage) {
                        webAppCore.showToast("Cannot pay below minimum wage!");
                        return;
                    }

                    currentGame.accountantSalary = wageInt;

                    webAppCore.showToast({
                        message: "HR Wage Set: " + numberToCash(amt),
                        image: "images/ui/accountant.png",
                        action: {
                            title: "Show",
                            func: showHR
                        }
                    });
                }
            });
        }
    }
    
    function workerSalaryDialog() {
        if(!webAppCore.getIsPremium()) {
            showPremiumDialog();
            return;
        }

        webAppCore.inputBox({
            title: "Salary",
            message: "Enter the new annual salary for our workers.  Current government mandated minimum salary is  " + numberToCash(currentGame.minimumWage) + ".",
            func: function(amt) {
                if(!amt || amt.trim().length == 0 || isNaN(amt)) {
                    webAppCore.showToast("Invalid Wage!");
                    return;
                }

                let wageInt = parseInt(amt);

                if(wageInt < currentGame.minimumWage) {
                    webAppCore.showToast("Cannot pay below minimum wage!");
                    return;
                }

                currentGame.workerSalary = wageInt;

                webAppCore.showToast({
                    message: "Worker Wage Set: " + numberToCash(amt),
                    image: "images/ui/worker.png",
                    action: {
                        title: "Show",
                        func: showHR
                    }
                });
            }
        });
    }

    function managerSalaryDialog() {
        if(!webAppCore.getIsPremium()) {
            showPremiumDialog();
            return;
        }

        webAppCore.inputBox({
            title: "Salary",
            message: "Enter the new annual salary for our managers.  Current government mandated minimum salary is  " + numberToCash(currentGame.minimumWage) + ".",
            func: function(amt) {
                if(!amt || amt.trim().length == 0 || isNaN(amt)) {
                    webAppCore.showToast("Invalid Wage!");
                    return;
                }

                let wageInt = parseInt(amt);

                if(wageInt < currentGame.minimumWage) {
                    webAppCore.showToast("Cannot pay below minimum wage!");
                    return;
                }

                currentGame.managerSalary = wageInt;

                webAppCore.showToast({
                    message: "Manager Wage Set: " + numberToCash(amt),
                    image: "images/ui/manager.png",
                    action: {
                        title: "Show",
                        func: showHR
                    }
                });
            }
        });
    }

    function inmateSalaryDialog() {
        if(!webAppCore.getIsPremium()) {
            showPremiumDialog();
            return;
        }

        webAppCore.inputBox({
            title: "Stipend",
            message: "Enter the new annual stipend for ex-employees doing time.",
            func: function(amt) {
                if(!amt || amt.trim().length == 0 || isNaN(amt)) {
                    webAppCore.showToast("Invalid Stipend!");
                    return;
                }

                let wageInt = parseInt(amt);

                if(wageInt < 0) {
                    webAppCore.showToast("Cannot pay negative!");
                    return;
                }

                currentGame.inmateSalary = wageInt;

                webAppCore.showToast({
                    message: "Inmate Stipend Set: " + numberToCash(amt),
                    image: "images/ui/inmate.png",
                    action: {
                        title: "Show",
                        func: showHR
                    }
                });
            }
        });
    }

    function addAccountant() {
        currentGame.AccountantCount += 1;

        let accountingBudget = currentGame.AccountantCount * currentGame.accountantSalary;

        webAppCore.showAlertWithTitle("You have added an accountant to the staff.  Annual costs at the accounting department are now " + numberToCash(accountingBudget) + ".");
    }

    function playNextSong() {
        if(allowAudio) {

            let nextSong = currentSong;

            while(nextSong == currentSong) {
                nextSong = ManagementGame.soundtrack[randomIntFromInterval(0,ManagementGame.soundtrack.length - 1)];
            }

            currentSong = nextSong;

            webAppCore.playMidi(currentSong.name.split(".")[0],currentSong.length,false,onMidiStatus);
        } else {
            currentSong = null;
            webAppCore.stopMidi();
        }
    }

    function randomIntFromInterval(min,max) {
        return webAppCore.randomIntFromInterval(min,max);
    }

    function onMidiStatus(status) {
        if(status == "done" && allowAudio) {
            currentSong = null;
            playNextSong();
        }
    }

    function getNewLoanStatus() {
        let totalSellValueWorth = 0;
        let i;

        for(i = 0; i < currentGame.businesses.length; i++) {
            let sellValue = getBusinessRevenuePerDay(currentGame.businesses[i]) * 8;
        
            if(currentGame.businesses[i].manager == 0) {
                sellValue = Math.floor(sellValue / 4);
            }
            
            if(currentGame.businesses[i].sellValue && currentGame.businesses[i].sellValue > 0) {
                sellValue = currentGame.businesses[i].sellValue;
            }

            if(isNaN(sellValue)) {
                sellValue = 0;
            }

            totalSellValueWorth += sellValue;
        }

        totalSellValueWorth += parseInt(currentGame.cash);
        totalSellValueWorth += parseInt(currentGame.accounting.revenue.quarter);

        if(totalSellValueWorth < 1000) {
            totalSellValueWorth = 1000;
        }

        totalSellValueWorth *= 2.5;

        pendingMaxLoanAmount = parseInt(totalSellValueWorth);

        let dialogOptions = {
            title: "New Business Loan",
            message: "Your loan officer at The Bank has been authorized to lend you up to " + numberToCash(totalSellValueWorth) + ".  How much do you want to borrow today?",
            func: onLoanAmountInput
        };
        
        webAppCore.inputBox(dialogOptions);

    }

    function onLoanAmountInput(amount) {

        if(!amount) {
            return;
        }

        if(isNaN(amount)) {
            return;
        }

        let rawAmount = parseInt(amount);

        if(rawAmount < 1) {
            return;
        }

        if(rawAmount > pendingMaxLoanAmount) {
            webAppCore.showAlert("Sorry, we cannot loan you that much!");
            return;
        }

        let option1 = buildLoan("One Year Business Loan",rawAmount,12);
        let option2 = buildLoan("Three Year Business Loan",rawAmount,36);
        let option3 = buildLoan("Five Year Business Loan",rawAmount,60);
        let option4 = buildLoan("Ten Year Business Loan",rawAmount,120);

        let loanOptions = [];

        loanOptions.push({
            title: option1.name,
            description: numberToCash(option1.balance) + " at " + (option1.rate * 100).toFixed(2) + "%",
            status: numberToCash(option1.termPayment) + "/mo",
            tag: option1
        });

        loanOptions.push({
            title: option2.name,
            description: numberToCash(option2.balance) + " at " + (option2.rate * 100).toFixed(2) + "%",
            status: numberToCash(option2.termPayment) + "/mo",
            tag: option2
        });

        loanOptions.push({
            title: option3.name,
            description: numberToCash(option3.balance) + " at " + (option3.rate * 100).toFixed(2) + "%",
            status: numberToCash(option3.termPayment) + "/mo",
            tag: option3
        });

        loanOptions.push({
            title: option4.name,
            description: numberToCash(option4.balance) + " at " + (option4.rate * 100).toFixed(2) + "%",
            status: numberToCash(option4.termPayment) + "/mo",
            tag: option4
        });

        webAppCore.presentNativeList("Choose Financing",loanOptions,onFinalLoanSelection,"Nevermind");
    }

    function onFinalLoanSelection(loan) {
        if(!loan) {
            return;
        }

        modifyCashAmount(loan.balance,false);
        currentGame.loans.push(loan);

        if(allowAudio) {
            webAppCore.playSoundEffect("chaching");
        }
    }

    function showLegalScreen() {
        let legalOptions = [];

        let legalBudget = currentGame.lawyerCount * currentGame.lawyerSalary;

        let performanceOverflow = null;

        if(currentGame.chargesDropped > 0) {
            performanceOverflow = [{
                pre:    "Charges Dropped: ",
                post:   numberWithCommas(currentGame.chargesDropped)
            }];
        }

        legalOptions.push({
            title: "Legal Department",
            description: "You have " + numberWithCommas(currentGame.lawyerCount) + " full time lawyers on staff.  Lawyers reduce your legal fines, jail sentences (should it come to that), or prevent 'the man' from ever getting to you in the first place.",
            icon: "images/ui/lawyer.png",
            status: "Annual Expenses: " + numberWithCommas(legalBudget),
            overflow: performanceOverflow,
            tag: "staff"
        });

        let rapSheet = [];

        if(currentGame.rapFinesPaid > 0) {
            rapSheet.push({
                pre: "Fines Paid: ",
                post: numberToCash(currentGame.rapFinesPaid)
            });
        }

        if(currentGame.rapBusinessesRaided > 0) {
            rapSheet.push({
                pre: "Businesses Raided: ",
                post: numberWithCommas(currentGame.rapBusinessesRaided)
            });
        }

        for(let crime in currentGame.criminalCharges) {
            rapSheet.push({
                pre: crime + ": ",
                post: numberWithCommas(currentGame.criminalCharges[crime]) + " counts"
            });
        }

        if(currentGame.daysServed > 0) {
            rapSheet.push({
                pre: "Time Served: ",
                post: numberWithCommas(currentGame.daysServed) + " days"
            });
        }

        if(rapSheet.length > 0) {
            legalOptions.push({
                title: "Criminal Record",
                description: "Here is your current history of run-ins with the law.",
                icon: "images/ui/rapsheet.png",
                overflow: rapSheet
            });
        }
        

        webAppCore.presentNativeList("Legal",legalOptions,onLegalItemSelected,"Dismiss");
    }

    function onLegalItemSelected(tag) {
        if(!tag) {
            return;
        }

        if(tag == "staff") {
            legalStaffingDialog();
        }
    }

    function showResources() {
        let resourceItems = [];

        

        let totalValue = 0;

        for(let res in currentGame.resources) {
            let resource = currentGame.resources[res];

            if(resource.isUnlocked) {
                let rDef = getResourceDefByName(res);

                resourceItems.push({
                    title: res,
                    description: "Inventory: " + numberWithCommas(resource.inv),
                    status: "Market Price/Unit: " + numberToCash(resource.price),
                    overflow: [
                        {
                            pre: "Total Value: ",
                            post: numberToCash(resource.inv * resource.price)
                        }
                    ],
                    icon: "images/resources/" + rDef.icon,
                    tag: rDef
                });

                totalValue += (resource.inv * resource.price);
            }
        }

        let storStatus = currentGame.currentStorage + "/" + currentGame.storageCap;

        let percent =  Math.round((currentGame.currentStorage / currentGame.storageCap) * 100);

        storStatus += " (" + percent + "%)";

        let storStatusColor = null;
        if(percent > 90) {
            storStatusColor = "#ff0000";
        }

        resourceItems.unshift({
            title: "Storage",
            description: "The total amount of resources you can hold.  Buy storage facilities to hold more resources.",
            status: storStatus,
            "status-color": storStatusColor,
            progress: percent,
            icon: "images/ui/storage.png",
            overflow: [
                {
                    pre: "Total Value: ",
                    post: numberToCash(totalValue)
                }
            ]
        });

        webAppCore.presentNativeList("Resources",resourceItems,onResourceSelected,"Dismiss");
    }

    function onResourceSelected(tag) {
        if(!tag) {
            return;
        }

        if(!currentGame.resources[tag.name]) {
            currentGame.resources[tag.name] = new Resource(tag);
        }

        if(!tag.canSell) {
            return;
        }

        let res = currentGame.resources[tag.name];

        if(res.inv <= 0) {
            webAppCore.showToast({
                message: "You have no " + tag.name,
                image: "images/resources/" + tag.icon
            });
            return;
        }

        tmpSellResVal = 1;

        webAppCore.showDialog({
            title: "Sell " + tag.name,
            message: "You can sell " + tag.name + " for " + numberToCash(res.price) + " per unit.  You have " + res.inv + " in storage.<br /><br />How much would you like to sell?",
            icon: "images/resources/" + tag.icon,
            sliderMin: 0,
            sliderMax: res.inv,
            sliderValue: 1,
            sliderStatusText: numberToCash(res.price),
            sliderCallback: function(val,func) {
                tmpSellResVal = parseInt(val);
                let totPrice = res.price * tmpSellResVal;

                func(tmpSellResVal + " for " + numberToCash(totPrice));
            },
            buttons: [
                {
                    text: "Sell",
                    color: "#4CAF50",
                    func: function(){
                        if(tmpSellResVal <= 0) {
                            return;
                        }

                        let totPrice = res.price * tmpSellResVal;

                        modifyResourceAmount(tag.name,-tmpSellResVal);
                        modifyCashAmount(totPrice,tag.contraband);

                        if(allowAudio) {
                            webAppCore.playSoundEffect("chaching");
                        }

                        showResources();
                    }
                },
                {
                    text: "Cancel",
                    func: showResources
                }
            ]
        });
    }

    function processInmateDay() {

        let freedInmates = [];
        let snitches = [];

        let i;

        for(i = 0; i < currentGame.jailedWorkers.length; i++) {
            let worker = currentGame.jailedWorkers[i];

            if(worker.days > 0) {
                worker.days--;
            }

            if(worker.days < 0) {
                worker.days = 0;
            }

            if(worker.fine > 0) {
                if(worker.wallet > 0) {
                    worker.fine -= worker.wallet;
                    worker.wallet = 0;
                }

                if(worker.fine < 0) {
                    worker.fine = 0;
                }
            }

            if(worker.fine == 0 && worker.days == 0) {
                freedInmates.push(worker);
            } else {
                worker.mood -= randomIntFromInterval(0,1);

                if(worker.mood < 40) {
                    let snitchChance = randomIntFromInterval(0,200);
                    if(snitchChance == 15) {
                        snitches.push(worker);
                    }
                }
            }

            
        }

        for(i = 0; i < freedInmates.length; i++) {
            removeFromArray(currentGame.jailedWorkers,freedInmates[i]);
        }

        let totalFine = 0;
        let criminalCounts = {};

        let counts = 0;

        for(i = 0; i < snitches.length; i++) {
            removeFromArray(currentGame.jailedWorkers,snitches[i]);

            totalFine += snitches[i].fine;

            if(snitches[i].crime) {
                if(criminalCounts[snitches[i].crime]) {
                    criminalCounts[snitches[i].crime]++;
                } else {
                    criminalCounts[snitches[i].crime] = 1;
                }
                counts++;
            }
        }

        if(!playingCatchup && (totalFine > 0 || counts > 0)) {
            // for each lawyer, a 10% chance of charges being dropped
            for(i = 0; i < currentGame.lawyerCount; i++) {
                let dropChance = randomIntFromInterval(0,100);
                if(dropChance < 10) {
                    currentGame.chargesDropped++;
                    return;
                }
            }
        }

        if(totalFine == 0 && counts == 0) {
            return;
        }

        let totalTime = 0;

        let crime;

        if(counts > 0) {
            

            for(crime in criminalCounts) {
                let crimeDef = getCrimeByName(crime);

                totalFine += crimeDef.fine;
                totalTime += crimeDef.time;
            }
        }

        let lawyerReductionChance = 0.05 * currentGame.lawyerCount;
        if(lawyerReductionChance > 0.75) {
            lawyerReductionChance = 0.75;
        }

        totalFine = randomIntFromInterval(0,totalFine - Math.round(totalFine * lawyerReductionChance));
        totalTime = randomIntFromInterval(0,totalTime - Math.round(totalTime * lawyerReductionChance));

        let msg = "Bad news!  Former employees currently serving time got reduced sentences by ratting us out to the District Attorney!";
        let titleMsg = "Fine Imposed!";
        let theImg = "images/ui/money.png";

        if(totalFine > 0) {
            msg += "  We have been fined " + numberToCash(totalFine) + "!";
        }

        let counter = 0;

        if(totalTime > 0) {

            titleMsg = "Under Arrest!";
            theImg = "images/ui/inmate.png";

            currentGame.jailDaysRemaining += totalTime;

            msg += "  You have been sentenced to " + totalTime + " days in prison for";

            for(crime in criminalCounts) {

                if(counter > 0) {
                    msg += ",";
                }

                msg += " " + crime;

                if(criminalCounts[crime] > 1) {
                    msg += " x" + criminalCounts[crime];
                }

                counter++;
            }

            msg += "!";
        }

        if(totalFine > 0) {
            modifyCashAmount(-totalFine,false);

            if(currentGame.cash < 0) {
                let loanAmount = -currentGame.cash;
                originateLoan("Y" + (currentGame.year + 1).toString() + "M" + (currentGame.month + 1).toString() + " Legal Fees Loan",loanAmount,12);
                msg += "  Accounting was forced to take an emergency one year bank loan to finance your legal expenses!";
            }
        }

        webAppCore.showDialog({
            title: titleMsg,
            message: msg,
            image: theImg
        });

    }

    function getStackPrice(business) {
        let luc = business.levelUpCost;
        let muc = business.managerUpCost;
        let bdef = getBusinessDefByName(business.name);

        let totalCost = bdef.cost + luc + muc;

        return totalCost;
    }
})();